/*
    Boostbloader.
    
    Based on Bloader by Sparkfun
 
	This version - Ian Harris imharris at gmail.com
 
	See changes.txt for changes
 
*/
 
 
#include <system.h>
#include "pic_utils.h"
#include "chipdefs.h"
#include "config.h"
#include "configbits.h"

#include "baud_rate_config.h"
 
// Pre-declare functions
 
void putc(uns8 nate);
uns8 getc(void);
 
void onboard_program_erase(uns16 program_address);
void onboard_program_write(void);
 
// Global variables
 
uns8 incoming_buffer[16]; //Presume no record will be longer than 16 bytes
uns16 memory_address;
uns8 record_length;
 
 
// The PIC sends ASC(5)-enquiry out and waits for return ASC(6)-ack from computer.
// If ACK is seen, PIC goes into load mode, otherwise, returns to main 
// The PIC will transmit a 'T' and wait for a ':' to start listening
 
void main(void)
{
 
    uns8 i, temp;
    uns8 check_sum = 0;
 
#ifdef _PIC16F88
    osccon = 0b01110000;  // Setup internal oscillator for 8MHz
    while(osccon.2 == 0); // Wait for frequency to stabilize
#endif
#if PLATFORM_TYPE == SURE_PICDEM_2
	trisb = 0b00000000;
	latb =  0b00001110;	// Turn on a led on the dev board
#endif
#if PLATFORM_TYPE == OLIMEX_BOARD
	trisa = 0b00000000;
	#ifdef _PIC16
		porta.0 = 1;	// Turn on a led on the dev board
	#endif
	#ifdef _PIC18
		lata = 0x01;
	#endif	
#endif
#if PLATFORM_TYPE == SFE_TDN_V1
	trisb = 0b00000000;
	portb.1 = 1;
#endif
#if PLATFORM_TYPE == CLOCK_BOARD
	trisc = 0b00000000;
	#ifdef _PIC18
		latc.1 = 1;
	#endif	
#endif
 
#ifdef _PIC18F26K20
	#warning "A 18f26k20"
	#ifdef OSC_INTRC
		osccon = 0b01110000;  // Setup internal oscillator for 16MHz
		while(osccon.2 == 0); // Wait for frequency to stabilize
	#endif
#endif	
	 
#ifdef PLL_ENABLED_IN_SW
	set_bit(osctune, PLLEN);
#endif	 
 
 
	// This routine comes from chipdefs.h
	SERIAL_TRIS_SETUP();
	spbrg = SPBRG_9600;	// Automatically set for 9600 bps
	rcsta = 0b10010000; // Serial port enable, 8-bit asych continous receive mode
    #ifdef BRGH_OFF
		#warning "high speed uart disabled"
		txsta = 0b00100000; // 8-bit asych mode, high speed uart disabled
	#else
		txsta = 0b00100100; // 8-bit asych mode, high speed uart enabled
	#endif
	
    for(i = 0 ; i < 255 ; i++) {
		asm {
			nop
			nop
			nop
			nop
			nop
			nop
		}	
	}
    putc(5); // PIC tells the world it can be loaded, by broadcasting an Enquiry
 
	set_bit( t1con, T1CKPS1);	// presecaler - slow things down a bit
	set_bit( t1con, T1CKPS0);	// presecaler - slow things down a bit
    tmr1l = 0; 
    tmr1h = 0;
	set_bit( t1con, TMR1ON); //enable timer 1
	uns8 count = 0;
    while(!test_bit(pir1, RCIF)) // Check to see if the computer responded
        if (test_bit(pir1, TMR1IF)) {
        	count++;
        	clear_bit(pir1, TMR1IF);
        	if (count == 5) {
				break; // If timer1 rolls over, go to normal boot up
			}	
        }	
	clear_bit( t1con, TMR1ON ); // timer 1 off
    if (rcreg != 6) goto JUMP_VECTOR; // If the computer did not respond correctly with a ACK, we go into normal boot up mode

    while(!test_bit(pir1,RCIF)); // Wait for computer to transmit UART speed
    
    #ifdef BRGH_OFF
		txsta = 0b00100100; // turn high speed uart back on
	#endif


    spbrg = rcreg; // Setup the hardware UART module
    
    //Short delay while computer re-adjusts the port speed under VB
    //for(i = 0 ; i < 255 ; i++)
    //    for(temp = 0 ; temp < 255 ; temp++);
 
    // Erase memory block -> 0 to almost the start of the boot loader
#ifdef erase_before_write
	// Erase first block
	onboard_program_erase(0x0000);
    // The boot vector has been erased, so write it back!
 
	#ifdef _PIC16
    incoming_buffer[0] = 0x15; // - This sets the far jump
    incoming_buffer[1] = 0x8A;
 
    incoming_buffer[2] = 0x16; // - More far jump
    incoming_buffer[3] = 0x0A;
 
    incoming_buffer[4] = 0x2E;
    incoming_buffer[5] = BLOADER_START_LOW; 
    // IH: No longer need location 6,7 since we move the program's boot vector
    record_length = 8;
	#endif
 
	#ifdef _PIC18
 
    incoming_buffer[0] = JUMP_TO_BLOADER1; //0xef; // Goto bloader
    incoming_buffer[1] = JUMP_TO_BLOADER2; //0x04;
 
    incoming_buffer[2] = JUMP_TO_BLOADER3; //0xf0; 
    incoming_buffer[3] = JUMP_TO_BLOADER4; //0x3f;
 
    record_length = 4;
	#endif
 
    memory_address = 0;
    onboard_program_write();
 
	// Now erase the rest
    for(memory_address = 0x0000+min_erase_chunk; 
        memory_address  < BLOADER_START - min_erase_chunk - 1;
        memory_address += min_erase_chunk)
			onboard_program_erase(memory_address);
 
 
#endif // erase before write
 
	// This is only necessary for pic 16 devices
	// Since they need to clear the pclath before jumping
	// this needs to be done since we have relocated the first 4
	// bytes of the program - normally a jump to main, startup
	// etc.  We can't rely on the compiler to put the pclath clearing
	// in it, since the compiler is assuming it's boot time, and hence
	// the pclath is already clear.
	//
	// pic 18 jumps directly, so it is not required
	//
	#ifdef _PIC16
	incoming_buffer[0] = 0x12; // - 120A  	BCF PCLATH,4
	incoming_buffer[1] = 0x0A;
 
	incoming_buffer[2] = 0x11; // - 118A  	BCF PCLATH,3
	incoming_buffer[3] = 0x8A;
 
	incoming_buffer[4] = 0x00; // - Nop to make up four instructions
	incoming_buffer[5] = 0x00;
 
	incoming_buffer[6] = 0x00; // - nop to make up four instructions
	incoming_buffer[7] = 0x00; 
 
    memory_address = MOVED_BVECTOR_START;
    record_length = 8;
    onboard_program_write();
	#endif
 
    //=====================================================
 
    // IH: used to send bloader location, now we need to send moved boot vector location
    //     since this is lower
    
    putc(MOVED_BVECTOR_HIGH); // This will tell the computer how to remap the boot vector
    putc(MOVED_BVECTOR_LOW); // This will avoid writing into restricted bloader code space
 
    while(1)
    {
        // Determine if the last received data was good or bad
        if (check_sum != 0) // If the check sum does not compute, tell computer to resend same line
            putc(7); // Ascii character BELL
        else            
            putc('T'); // Tell the computer that we are ready for the next line
        
        while(getc() != ':'); // Wait for the computer to initiate transfer
 
        record_length = getc(); // Get the length of this block
 
        if (record_length == 'S') goto JUMP_VECTOR; // Check to see if we are done - new boot vector
 
        memory_address = getc();
        memory_address <<= 8;
        memory_address += getc();
        check_sum = getc(); // Pick up the check sum for error dectection
        
        for(i = 0 ; i < record_length ; i++) // Read the program data
        {
            temp = getc();
            incoming_buffer[i] = temp;
        }
        
        for(i = 0 ; i < record_length ; i++) // Check sum calculations
            check_sum = check_sum + incoming_buffer[i];
        
        check_sum = check_sum + record_length;
        check_sum = check_sum + (memory_address >> 8);
        check_sum = check_sum + (memory_address & 0xff);
 
        #ifdef _PIC18
			memory_address <<= 1;
        #endif
        
        if(check_sum == 0) { // If we have a good transmission, put it in ink
			if ( memory_address == 0 ) {  // IH: If it's the boot vector
				#ifdef _PIC16
				// For pic_16, skip over the pclath instructions
				memory_address = MOVED_BVECTOR_START + 4;
				#endif
				#ifdef _PIC18
				// For pic_18, just relocate into the moved_bvector
				memory_address = MOVED_BVECTOR_START;
				#endif
            } 
                  
            onboard_program_write();
 
        }            
 
    }
JUMP_VECTOR:
#if PLATFORM_TYPE == SURE_PICDEM_2
	latb =  0b00001111;	// Turn off the led
#endif
#if PLATFORM_TYPE == OLIMEX_BOARD
	#ifdef _PIC16
		porta.0 = 0;	// Turn off the dev board led
	#endif
	#ifdef _PIC18
		#warning "led off"
		lata = 0x00;		// Turn off the dev board led
	#endif	
#endif
#if PLATFORM_TYPE == SFE_TDN_V1
	portb.1 = 0;
#endif
#if PLATFORM_TYPE == CLOCK_BOARD
	#ifdef _PIC18
		#warning "led off"
		latc.1 = 0;		// Turn off the dev board led
	#endif	
#endif
 
    clear_bit(txsta, TXEN); //Disable transmission
    clear_bit(txsta, CREN); //Disable receiver
    
 //    IH: Now - do a jump to where the vector is, and it's in the 
 //        same memory page as we are, so no need to change PCLATH
 //		   for PIC16 devices
  
  #ifdef _PIC16
  asm data JUMP_TO_BVECTOR // goto MOVED_BVECTOR_START
  #endif
  
  #ifdef _PIC18
  asm data JUMP_TO_BVECTOR1  // goto MOVED_BVECTOR_START
  asm data JUMP_TO_BVECTOR2
  #endif  
 
}
 
 
 
// Erase one block of program memory
 
#ifdef _PIC16
 
void onboard_program_erase(uns16 program_address)
{
 
    HIBYTE(eeadrh, program_address); // set the address
    LOBYTE(eeadr, program_address); // set the address
 
    set_bit(eecon1, EEPGD); // point to Program data block
    #ifdef FREE
    set_bit(eecon1, WREN); // enable EE Writes
    set_bit(eecon1, FREE); // allow program memory changes
    #endif
    // ppecific Program Erase/EEPROM write steps
    eecon2 = 0x55;
    eecon2 = 0xAA;
    set_bit(eecon1, WR);
 
    // processor stalls and resumes after second NOP
    nop();
    nop();
 
    clear_bit(eecon1, WREN);
} 
 
#endif
 
#ifdef _PIC18
void onboard_program_erase(uns16 program_address)
{
	tblptru = 0;
	HIBYTE(tblptrh, program_address); //Set the address
    LOBYTE(tblptrl, program_address); //Set the address
 
    set_bit(eecon1, EEPGD); //Point to Flash Program data block
    clear_bit(eecon1, CFGS); // Access Flash program memory
    set_bit(eecon1, WREN); //Enable EE Writes
    set_bit(eecon1, FREE); //Allow program memory changes
 
    //Specific Program Erase/EEPROM write steps
    eecon2 = 0x55;
    eecon2 = 0xAA;
    set_bit(eecon1, WR);
 
    clear_bit(eecon1, WREN);
} 
#endif
 
 
#ifdef _PIC16
//Write e_data to the onboard eeprom at e_address
void onboard_program_write(void)
{
    uns8 i;
    
#ifdef OSCCON
    clear_bit(eecon1, FREE);
#endif
    set_bit(eecon1, EEPGD); //Point to Program data block
    set_bit(eecon1, WREN); //Enable EE Writes
 
    HIBYTE(eeadrh, memory_address); //Set the address
    LOBYTE(eeadr, memory_address); //Set the address
    //EEADR = EEADR & 0b.1111.1100; //Make sure address is 0 of four bytes
 
    for(i = 0 ; i < record_length ; i += 2)
    {
       eedath = incoming_buffer[i]; //Give it the data
       eedata = incoming_buffer[i+1]; //Give it the data
 
        //Specific EEPROM write steps
        eecon2 = 0x55;
        eecon2 = 0xAA;
        set_bit(eecon1, WR);
        //Specific EEPROM write steps
    
        //Processor stalls and resumes after second NOP
        nop();
        nop();
        
        eeadr++; //Go to next buffer spot
        if (eeadr == 0) eeadrh++;
    }
    
    clear_bit(eecon1, WREN); //Disable EEPROM Writes
 
}  
#endif
 
#ifdef _PIC18
 
#if max_write_chunk < 16 
 
void write_flash() {
	set_bit  (eecon1, EEPGD); //Point to Program data block
    clear_bit(eecon1, CFGS); // Access Flash program memory
    set_bit  (eecon1, WREN); //Enable EE Writes
 
    //Specific EEPROM write steps
    eecon2 = 0x55;
    eecon2 = 0xAA;
    set_bit(eecon1, WR);
    //Specific EEPROM write steps
 
	// Not required for 18f2620 ??
 
	clear_bit(eecon1, WREN); //Disable EEPROM Writes
}
 
void onboard_program_write(void)
{
    uns8 i;
    
	tblptru = 0;	// Assume no more than 64k
	HIBYTE(tblptrh, memory_address); //Set the address
    LOBYTE(tblptrl, memory_address); //Set the address
	asm TBLRD*- 	// dummy decrement
    for(i = 0 ; i < record_length ; i += 2)
    {
		// seem to need to swap the bytes around
		tablat = incoming_buffer[i+1]; //Give it the data
		asm TBLWT+*
		tablat = incoming_buffer[i]; //Give it the data
		asm TBLWT+*
		if (i == max_write_chunk - 2) {
			write_flash();
		}	
 
    }
	write_flash();
 
}  
 
#else
 
void onboard_program_write(void)
{
    uns8 i;
    uns8 block_count;
    
	tblptru = 0;	// Assume no more than 64k
	HIBYTE(tblptrh, memory_address); //Set the address
    LOBYTE(tblptrl, memory_address); //Set the address
 
    for(i = 0 ; i < record_length ; i += 2)
    {
		// seem to need to swap the bytes around
	   tablat = incoming_buffer[i+1]; //Give it the data
		asm TBLWT*+
	   tablat = incoming_buffer[i]; //Give it the data
		asm TBLWT*+
 
 
    }
 
	// dummy decrement
	// ensure we're still in the right 64 byte block
	asm TBLRD*-
 
    set_bit  (eecon1, EEPGD); //Point to Program data block
    clear_bit(eecon1, CFGS); // Access Flash program memory
    set_bit  (eecon1, WREN); //Enable EE Writes
 
    //Specific EEPROM write steps
    eecon2 = 0x55;
    eecon2 = 0xAA;
    set_bit(eecon1, WR);
    //Specific EEPROM write steps
 
	// Not required for 18f2620 ??
	clear_bit(eecon1, WREN); //Disable EEPROM Writes
 
}  
#endif
#endif
 
//Sends out RS232 characters
//Speed is controller by UART setup registers
void putc(uns8 nate)
{
    while(!test_bit(pir1, TXIF));
    txreg = nate;
}
 
//Reads in RS232 characters
//Speed is controller by UART setup registers
uns8 getc(void)
{
    while(!test_bit(pir1, RCIF));
    return(rcreg);
}
 
#pragma OPTIMIZE "0"
void do_nothing() @ MOVED_BVECTOR_START { 

asm {
	start:	
		goto start
	}
}		