#ifndef __config_h
#define __config_h
#include "platform.h"

// - - - - - - - - - - - - - - - - - - - - 
// TMP75 temp sensor defines
// - - - - - - - - - - - - - - - - - - - - 

#define TMP75_ADDR 0x00000000

// - - - - - - - - - - - - - - - - - - - - 
// pic_serial defines
// - - - - - - - - - - - - - - - - - - - - 

#define SERIAL_TX_BUFFER_SIZE 240
#define SERIAL_RX_BUFFER_SIZE 4
//#define SERIAL_DEBUG_ON
#define SERIAL_DISCARD_ON_TX_FULL_DURING_INT
// - - - - - - - - - - - - - - - - - - - - 
// pic_term defines
// - - - - - - - - - - - - - - - - - - - - 

#define TERM_BUFFER_SIZE	10
#define TERM_ALLOW_BOOSTBLOADER
#define TERM_ECHO_INPUT

// - - - - - - - - - - - - - - - - - - - - 
// sure 6432 defines
// - - - - - - - - - - - - - - - - - - - - 

/*

Here we define two displays that are aligned vertically,
ie, one below the other.
-------------
|    Disp   | 
|     1     |
-------------
-------------
|    Disp   | 
|     2     |
-------------

This makes a 64x64 pixel display.

*/

#define ea_ldp6432_displays 1
#define ea_ldp6432_display_orientation VERTICAL

#define ea_ldp6432_en_port PORTC
#define ea_ldp6432_en_pin  0

#define ea_ldp6432_r1_port PORTA
#define ea_ldp6432_r1_pin  5

#define ea_ldp6432_g1_port PORTB
#define ea_ldp6432_g1_pin  4

#define ea_ldp6432_r2_port PORTA
#define ea_ldp6432_r2_pin  0	

#define ea_ldp6432_g2_port PORTB
#define ea_ldp6432_g2_pin  5

#define ea_ldp6432_a_port PORTA
#define ea_ldp6432_a_pin  1

#define ea_ldp6432_b_port PORTA
#define ea_ldp6432_b_pin  2

#define ea_ldp6432_c_port PORTA
#define ea_ldp6432_c_pin  3

#define ea_ldp6432_d_port PORTA
#define ea_ldp6432_d_pin  4

#define ea_ldp6432_s_port PORTB
#define ea_ldp6432_s_pin  6

#define ea_ldp6432_l_port PORTB
#define ea_ldp6432_l_pin  7


// - - - - - - - - - - - - - - - - - - - - 
// Draw defines
// - - - - - - - - - - - - - - - - - - - - 

// y
#define DRAW_PIXELS_HIGH 32

// x
#define DRAW_PIXELS_WIDE 64

// red and green leds per pixel = 2 colours = 2 bits
#define DRAW_BITS_PER_PIXEL 2

#define DRAW_HW_Y_ORIGIN TOP_LEFT

#define DRAW_HW_BUFFER_ORIENTATION HORIZONTAL

#define DRAW_DEBUG


// - - - - - - - - - - - - - - - - - - - - 
// pic i2c defines
// - - - - - - - - - - - - - - - - - - - - 

// for on board i2c devices

#define i2c_sda_port PORTB
#define i2c_sda_pin  3
#define i2c_scl_port PORTB
#define i2c_scl_pin  2

// - - - - - - - - - - - - - - - - - - - - 
// ITS mode2 defines
// - - - - - - - - - - - - - - - - - - - - 

// #define ITS_EA { 0, 0, 0, 0, 0, 0, 0, 3 }
#define ITS_MODE2_ROUTER
// or
//#define ITS_MODE2_DEVICE

#define ITS2_SEEN_LIST_SIZE 10
#define ITS2_TX_QUEUE_SIZE 5
#define ITS2_MAX_HOP_COUNT 5
#define ITS2_RESEND_TICK_DELAY 100
#define ITS2_SEND_MAX_TRIES 2

// - - - - - - - - - - - - - - - - - - - - 
// its common defines
// - - - - - - - - - - - - - - - - - - - - 

#define ITS_MAX_KNOWN_DEVICES 5


#define ITS_USE_MRF24J50
//#define ITS_MODE_1
//#define ITS_MODE_2
//#define ITS_DEVICE_TYPE ITS_M1_MASTER
// #define ITS_DEVICE_TYPE ITS_M1_SLAVE
// #define ITS_EXTENDED_ADDRESS {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01}
#define ITS_NETWORK_ID 0xa1a2
//#define ITS_DEVICE_ID  0xb001
#define ITS_FIXED_CHANNEL	15

// - - - - - - - - - - - - - - - - - - - - 
// spi_hw defines
// - - - - - - - - - - - - - - - - - - - - 

#define SPI_HW_MASTER_MODE
// or
//#define SPI_HW_SLAVE_MODE

// In master mode, we need to define clock
//#define SPI_HW_MASTER_CLOCK_TMR2_DIV_2
//#define SPI_HW_MASTER_CLOCK_FOSC_DIV_64
#define SPI_HW_MASTER_CLOCK_FOSC_DIV_16
//#define SPI_HW_MASTER_CLOCK_FOSC_DIV_4

// mrf24j40

// interrupt (input)
#define mrf24j40_int_port PORTB
#define mrf24j40_int_pin  0

// Chip Select (output, active low)
#define mrf24j40_cs_port PORTC
#define mrf24j40_cs_pin  2

// Reset (output, active low, weak pull-up)
//#define mrf24j40_reset_port PORTA
//#define mrf24j40_reset_pin  3

// Wake (output, disabled by default)
//#define mrf24j40_wake_port PORTA
//#define mrf24j40_wake_pin  4

// Depending on your module
#define ENABLE_PA_LNA
#define MRF24J40MB
#define APPLICATION_SITE USA


// - - - - - - - - - - - - - - -
// General platform definitions
// - - - - - - - - - - - - - - -
#define PLATFORM_TYPE EA_LED_PANEL_DRIVER
#define PLATFORM_CLOCK 40000000
#include "platform.h"
#define PLATFORM_LEDS_FLASH_TICKS 100

// - - - - - - - - - - - - - - - - - - - - 
// Debug defines
// - - - - - - - - - - - - - - - - - - - - 

#define debug_serial
#define debug_on
//#define debug_high_on


#endif