//-----------------------------------------------------
// Pic Pack library
// 
// ea_dsp0401b_demo.c
//
// Demonstration routines for the Embedded Adventures
// DSP-0401B 1" LED alphanumeric display
//
// Ian Harris 20111
// www.embeddedadventures.com
//
//-----------------------------------------------------


// Pic Pack includes
#include "config.h"
#include "pic_utils.h"
#include "pic_serial.h"
#include "ccld.h"

// System includes
#include "string.h"
#include "stdlib.h"
#include "config.h"
#include "pic_term.h"
#include "pic_tick.h"
#include "pic_timer.h"


#include "ea_dsp0401b.h"

uns8 test_mode = 0;
uns16 marker_tick;
uns8 my_var, my_var2;
// Interrupt routine - - - - - - - - - -

void interrupt() {
	
		serial_handle_tx_isr();
		serial_handle_rx_isr();
		
		timer_handle_0_isr();			
}

void display_something() {

// data loaded in on rising edge
uns8 count;

	ccld_write_data_byte(my_var);
	ccld_write_data(my_var2);
	
	// data loaded, now load it into latch

	ccld_latch_data();
	
	// now enable display
	
	ccld_enable_display(1);
	
	
}



//
// term_entry_callback will be called when 
// user pressed <enter> on serial "terminal".
// term_buffer contains the characters (if any)
// that the user has entered.

void term_entry_callback(uns8 *term_buffer) {

uns8 data[5];
uns8 first_char;
	
	first_char = term_buffer[0];

	switch (first_char) {
		case 'a':
			ea_dsp0401b_set_cursor(0);
			ea_dsp0401b_print_str("YELLYELLYELL");
			ea_dsp0401b_display();
			ccld_enable_display(1);
			break;
		case 'b':
			ea_dsp0401b_set_cursor(0);
			ea_dsp0401b_print_str("RED RED RED ");
			ea_dsp0401b_display();
			ccld_enable_display(1);
			break;
		case 'c':
			ea_dsp0401b_set_cursor(0);
			ea_dsp0401b_print_str("BLUEBLUEBLUEBLUE");
			ea_dsp0401b_display();
			ccld_enable_display(1);
			break;
		case 't':
			serial_print_str("\n>> Test mode: ");
			test_mode = ~test_mode;
			if (test_mode) {
				marker_tick = tick_get_count();
				serial_print_str("ON\n");
			} else {
				serial_print_str("OFF\n");
			}	
			break;
		case 'd':
			serial_print_str("You requested command c\n");
			display_something();
			break;
		case 'n':
			//doit();
			//dsp0401b_print_str("1234");
			break;
		case 'q':
			// More complicated example
			// Let's set a variable based on input
			// You can type:
			// s05
			// And press enter, it will set "my_var" to this (in hex)
			if (strlen(term_buffer)==3) {
				my_var2 = atoui_hex(&term_buffer[1]);
				serial_print_str(" Set my_var to ");
				serial_print_int(my_var2);	// Of course, this could be anything
				serial_putc('\n');
			} else {
				serial_print_str("Must use sxx where xx is two hex digits\n");
			}
			break;
			case 's':
			// More complicated example
			// Let's set a variable based on input
			// You can type:
			// s05
			// And press enter, it will set "my_var" to this (in hex)
			if (strlen(term_buffer)==3) {
				my_var = atoui_hex(&term_buffer[1]);
				serial_print_str(" Set my_var to ");
				serial_print_int(my_var);	// Of course, this could be anything
				serial_putc('\n');
			} else {
				serial_print_str("Must use sxx where xx is two hex digits\n");
			}
			break;
		default:
			serial_print_str("Unknown command.\n");
			break;
	}	// switch		

}	// term_entry_callback


// configure_system
//
// Get the pic ready for everything we want to do

void configure_system() {
	
	// turn off interrupts just in case
	kill_interrupts();
	
	// kill those pesky analogue inputs
	turn_analog_inputs_off();	
	
	// set serial to run at 115200bps 
	serial_setup(SPBRG_115200);
	
	// initialise termainl
	term_init();
	
	// setup IO pins for dsp-0401B
	ea_dsp0401b_setup_io();
	
		// Timer 0 is used for ticks - 1ms at 64mhz (16mips)
	timer_setup_0(TIMER_16BIT_MODE, TIMER_PRESCALER_OFF, 0xffff - 16000 -1); 
	
	// tern peripheral interrupts on (ie, serial port)
	turn_peripheral_ints_on();
	
	// turn all interrupts on
	turn_global_ints_on();
}



void main() {
	
	configure_system();
	
	delay_ms(100);	// mostly so screamer bootloader has time to switch windows
	
	serial_print_str("\n\nTEST \n");
	serial_print_str( "\n<");
	
	// Print the time and date this file was compiled. Handy to know which
	// version you are running.
	
	serial_print_str(__TIME__);
	serial_putc(' ');
	serial_print_str(__DATE__);
	serial_print_str(">\n");

	serial_print_str("Commands: a,x,c,n,s00\n");

	// a,x,c,n just press the key and enter
	// s00 sets the variable "my_var" to the hex value you enter

	uns16 test_tick;
	uns16 led_data = 1;
	timer_start_0();
	
	
	for(;;) {

		term_process();
		
		if (test_mode) {
			test_tick = tick_get_count();	// find out what we're up to

			if (tick_calc_diff(marker_tick, test_tick) >= 50) {	
				led_data = led_data << 1;
				if (led_data == 0) {
					led_data = 1;
				}
				ccld_write_data(led_data);
				ccld_write_data(led_data);
				ccld_write_data(led_data);
				ccld_write_data(led_data);
				ccld_latch_data();
		
				ccld_enable_display(1);
				marker_tick = test_tick;
			}	
		}	
	}

	
}	// main

