//-----------------------------------------------------
//-----------------------------------------------------
// Pic Pack library
// 
// EA_DSP0801_TEST.c
//
// Demonstration routines for the Embedded Adventures
// DSP-0801 alphanumeric display
//
// Ian Harris 2011
// www.embeddedadventures.com
//
//-----------------------------------------------------
//-----------------------------------------------------


// Pic Pack includes
#include "config.h"
#include "pic_utils.h"
#include "pic_serial.h"
#include "ccld.h"

// System includes
#include "string.h"
#include "stdlib.h"
#include "config.h"
#include "pic_term.h"

#include "ea_dsp0801.h"

// Dummy variable to show we can set it 

uns8 my_var = 0;
uns8 my_var2 = 0;

// Interrupt routine - - - - - - - - - -

void interrupt() {
	
		serial_handle_tx_isr();
		serial_handle_rx_isr();
}

void display_something() {

// data loaded in on rising edge
uns8 count;

	ccld_write_data_byte(my_var);
	ccld_write_data_byte(my_var2);
	
	// data loaded, now load it into latch

	ccld_latch_data();
	
	// now enable display
	
	ccld_enable_display(1);
	
	
}



//
// term_entry_callback will be called when 
// user pressed <enter> on serial "terminal".
// term_buffer contains the characters (if any)
// that the user has entered.

void term_entry_callback(uns8 *term_buffer) {

uns8 data[5];
uns8 first_char;
	
	first_char = term_buffer[0];

	switch (first_char) {
		case 'a':
			ea_dsp0801_set_cursor(0);
			ea_dsp0801_print_str("SLOT 12*");
			ea_dsp0801_display();
			break;
		case 'b':
			ea_dsp0801_set_cursor(0);
			ea_dsp0801_print_str("STORMTST");
			ea_dsp0801_display();
			break;
		case 'v':
			ea_dsp0801_set_cursor(0);
			ea_dsp0801_print_str("RESERVED");
			ea_dsp0801_display();
			break;
		case 'x':
			uns8 mychar = 'G';
			uns16 trans = ea_dsp0801_translate(mychar);
			ccld_write_data(trans);
			ccld_latch_data();
			break;
		case 'c':
			serial_print_str("You requested command c\n");
			display_something();
			break;
		case 'n':
			serial_print_str("Send single char (s)\n");
			ccld_write_data_byte(my_var);
			//doit();
			//dsp0401b_print_str("1234");
			break;
		case 'f':
			serial_print_str("Fill!\n");
			ea_dsp0801_fill();
			ea_dsp0801_display();			
			//doit();
			//dsp0401b_print_str("1234");
			break;			
		case 'q':
			// More complicated example
			// Let's set a variable based on input
			// You can type:
			// s05
			// And press enter, it will set "my_var2" to this (in hex)
			if (strlen(term_buffer)==3) {
				my_var2 = atoui_hex(&term_buffer[1]);
				serial_print_str(" Set my_var to ");
				serial_print_int(my_var2);	// Of course, this could be anything
				serial_putc('\n');
			} else {
				serial_print_str("Must use sxx where xx is two hex digits\n");
			}
			break;
			case 's':
			// More complicated example
			// Let's set a variable based on input
			// You can type:
			// s05
			// And press enter, it will set "my_var" to this (in hex)
			if (strlen(term_buffer)==3) {
				my_var = atoui_hex(&term_buffer[1]);
				serial_print_str(" Set my_var to ");
				serial_print_int(my_var);	// Of course, this could be anything
				serial_putc('\n');
			} else {
				serial_print_str("Must use sxx where xx is two hex digits\n");
			}
			break;
		default:
			serial_print_str("Unknown command.\n");
			break;
	}	// switch		

}	// term_entry_callback


// configure_system
//
// Get the pic ready for everything we want to do

void configure_system() {
	
	//clear_bit(intcon2, RABPU);
	//set_bit(wpub, WPUB4);
	//set_bit(wpub, WPUB6);
	
	// turn off interrupts just in case
	kill_interrupts();
	
	// kill those pesky analogue inputs
	turn_analog_inputs_off();	
	
	// set serial to run at 115200bps 
	serial_setup(SPBRG_115200);
	
	// initialise termainl
	term_init();
	
	//tmp75_setup_io();
	// 12 bit temp, continue temp, shutdown=0
	// OS R1 R0 F1 F0 POL TM SD
	// 0  1  1  0  0  0   0  0   
	//tmp75_set_config(TMP75_ADDR, 0b01100000);
	
	ccld_setup_io();

	// tern peripheral interrupts on (ie, serial port)
	turn_peripheral_ints_on();
	
	// turn all interrupts on
	turn_global_ints_on();
}



void main() {
	
	configure_system();
	
	delay_ms(100);	// mostly so screamer bootloader has time to switch windows
	
	serial_print_str("\n\nTEST \n");
	serial_print_str( "\n<");
	
	// Print the time and date this file was compiled. Handy to know which
	// version you are running.
	
	serial_print_str(__TIME__);
	serial_putc(' ');
	serial_print_str(__DATE__);
	serial_print_str(">\n");

	serial_print_str("Commands: a,x,c,n,s00\n");

	// a,x,c,n just press the key and enter
	// s00 sets the variable "my_var" to the hex value you enter


	for(;;) {

		term_process();
	
	}

	
}	// main

