//-----------------------------------------------------
// Pic Pack library
// 
// ea_mod1005_rtc.c
//
// Demonstration routines showing the RTC functionality
// on the Embedded Adventures MOD-1005 board
//
// (c) Embedded Adventures 2010
// iharris [at] embeddedadventures.com
//
//
//-----------------------------------------------------

// Pic Pack includes
#include "config.h"
#include "pic_utils.h"
#include "pic_serial.h"
// System includes
#include "string.h"
#include "stdlib.h"
#include "m41t81s.h"
// Support for Timer 0
#include "pic_timer.h"
#include "platform_leds.h"

#include "pic_term.h"
#include "pic_tick.h"

uns8 minutes;
uns8 hours;
uns8 seconds;
uns8 day;
uns8 date;

uns8 month;
uns8 year;

bit seconds_trigger = 0;

void update_clock() {

	
	// grab the latest time from the RTC chip

	minutes = rtc_get_minutes();
	hours   = rtc_get_hours();
	seconds = rtc_get_seconds();
	day 	= rtc_get_dow();
	date 	= rtc_get_date();
	month 	= rtc_get_month();
	year 	= rtc_get_year();
	
	
}

void request_time() {

uns8 minutes, hours, seconds, day, date, month, year;
char buffer[10];

	minutes = rtc_get_minutes();
	hours   = rtc_get_hours();
	seconds = rtc_get_seconds();
	day 	= rtc_get_dow();
	date 	= rtc_get_date();
	month 	= rtc_get_month();
	year 	= rtc_get_year();
	
	if (hours > 9) {
		buffer[0] = '0' + hours / 10;
		buffer[1] = '0' + hours % 10;
	} else {
		buffer[0] = ' ';
		buffer[1] = '0' + hours;
	}
	buffer[2] = ':';
	
	buffer[3] = '0' + minutes / 10;
	buffer[4] = '0' + minutes % 10;
	
	buffer[5] = ':';

	buffer[6] = '0' + seconds / 10;
	buffer[7] = '0' + seconds % 10;
	buffer[8] = 0;

	serial_print_str("The time is ");
	serial_print_str(buffer);
	
	serial_print_nl();
	
	serial_print_str("The day is ");
	
	switch (day) {
		case 1 : serial_print_str("Mon"); break;
		case 2 : serial_print_str("Tues"); break;
		case 3 : serial_print_str("Wednes"); break;
		case 4 : serial_print_str("Thurs"); break;
		case 5 : serial_print_str("Fri"); break;
		case 6 : serial_print_str("Satur"); break;
		case 7 : serial_print_str("Sun"); break;
	}
	serial_print_str("day\n");
	
	serial_print_str("The date is ");
	
	serial_print_int(date);
	serial_print_str(" of ");
	
	switch (month) {
		case 1 : serial_print_str("Jan"); break;
		case 2 : serial_print_str("Feb"); break;
		case 3 : serial_print_str("Mar"); break;
		case 4 : serial_print_str("Apr"); break;
		case 5 : serial_print_str("May"); break;
		case 6 : serial_print_str("Jun"); break;
		case 7 : serial_print_str("Jul"); break;
		case 8 : serial_print_str("Aug"); break;
		case 9 : serial_print_str("Sep"); break;
		case 10: serial_print_str("Oct"); break;
		case 11: serial_print_str("Nov"); break;
		case 12: serial_print_str("Dec"); break;
	}
	
	serial_print_str(" 20");
	if (year < 10) {
		serial_putc('0');
	}	
	serial_print_int(year);	
		
}



// Interrupt routine - - - - - - - - - -
void interrupt() {
	


	serial_handle_tx_isr();
	serial_handle_rx_isr();

	// Handle trigger to update clock every second
	if (test_bit(intcon3, INT2IF)) {
		clear_bit(intcon3, INT2IF);
		seconds_trigger = 1;
	}
	// Handle timer 0 interrupt
	timer_handle_0_isr();	
	
}



// Timer 0 callback handled by pic_tick

void term_entry_callback(uns8 *term_buffer) {

uns8 data[5];
uns8 first_char;
	
	first_char = term_buffer[0];
	switch (first_char) {
	case 't':
		request_time();
		break;
	case '2':
		
		break;
	case '3':
		
		break;
	case 'a':
		
		break;
	case 'b':
		break;
		
	case 'c':
		break;
	case 'd':
		break;
	case 'u':
		break;
	case 'p':
		break;
	default:
					if ((first_char == 's') && (strlen(term_buffer)>2)) {
					uns8 setting = atoui_dec(&term_buffer[2]);
					serial_print_str(" Setting to ");
					serial_print_int(setting);
					serial_putc(' ');
					switch (term_buffer[1]) {
					case 's' :
						rtc_set_seconds(setting);
						serial_print_str("sec");
						break;
					case 'm' :
						rtc_set_minutes(setting);
						serial_print_str("min");
						break;
					case 'h' :
						rtc_set_hours(setting);
						serial_print_str("hour");
						break;
					case 'a' :
						rtc_set_day(setting);
						serial_print_str("dow");
						break;
					case 't' :
						rtc_set_date(setting);
						serial_print_str("date");
						break;
					case 'o' :
						rtc_set_month(setting);
						serial_print_str("month");
						break;
					case 'y' :
						rtc_set_year(setting);
						serial_print_str("year");
						break;
					default:
						serial_print_str(" huh?");		
					}
					serial_putc('\n');
				}

	} // switch		

}

	
// configure_system
//
// Get the pic ready for everything we want to do

void configure_system() {
	    
	kill_interrupts();	// turn off interrupts just in case

	turn_analog_inputs_off();	// kill those pesky analogue inputs

	platform_leds_setup_io();


	// real time clock setup ----------------------------

	rtc_setup_io();

	rtc_start_clock();	// unpause the m41t81s
	rtc_set_sqw_freq(rtc_sqw_freq_1Hz);	// give us a shove every second
	rtc_start_sqw_output();	// turn on the 1Hz output
	
	make_input(PORTC, 2);	// sqw wired to portc.2
	clear_bit(intcon3, INT2IF);
	set_bit(intcon3, INT2IE);		// enable int1
	
	
	serial_setup(SPBRG_115200);

	term_init();	// initialise terminal 

	// LED setup ----------------------------
	
	platform_leds_setup_io();
	
	// general setup ----------------------------

	// Timer 0 is used for ticks - 1ms at 40mhz (10mips)
	timer_setup_0(TIMER_16BIT_MODE, TIMER_PRESCALER_OFF, 0xffff - 10000 -1); 

	turn_peripheral_ints_on();
	turn_global_ints_on();

}




void main() {

uns8 test_result, test_count;
uns16 marker_tick, test_tick;

	configure_system();
	delay_ms(100);

	serial_print_str("\n\nPicPack MOD-1005 demo\n");

	serial_print_str( "\n<");
	serial_print_str(__TIME__);
	serial_putc(' ');
	serial_print_str(__DATE__);
	serial_print_str(">\n");
	
	timer_start_0();

	
	test_count = 0;
	marker_tick = tick_get_count();	// find out what we're up to

	for(;;) {	// for ever
		
		term_process();	// process any typing on the terminal
			
		if (seconds_trigger) {
			update_clock();
			seconds_trigger = 0;
			platform_leds_flash(1);
		}
		
		test_tick = tick_get_count();	// find out what we're up to

		if (tick_calc_diff(marker_tick, test_tick) >= 1000) {	// 1000 - it's a second, 2000 = 2 seconds
			serial_print_str("Test\n");
			marker_tick = test_tick;
		}
		
		start_crit_sec();
			platform_leds_process();
		end_crit_sec();


		

	}
	
}	// main

