//-----------------------------------------------------
// Pic Pack library
// 
// ea_mod1008_humidity.c
//
// Demonstration routines showing the humidity and temperature sensors
// on the Embedded Adventures MOD-1008 board
//
// (c) Embedded Adventures 2011
// iharris [at] embeddedadventures.com
//
//
//-----------------------------------------------------

// Pic Pack includes
#include "config.h"
#include "pic_utils.h"
#include "pic_serial.h"
// System includes
#include "string.h"
#include "stdlib.h"
#include "sht15.h"
// Support for Timer 0
#include "pic_timer.h"
#include "platform_leds.h"

#include "pic_term.h"
#include "pic_tick.h"



// Interrupt routine - - - - - - - - - -
void interrupt() {
	
	serial_handle_tx_isr();
	serial_handle_rx_isr();

	// Handle timer 0 interrupt
	timer_handle_0_isr();	
	
}



// Timer 0 callback handled by pic_tick

void term_entry_callback(uns8 *term_buffer) {

uns8 data[5];
uns8 first_char;
	
	first_char = term_buffer[0];
	switch (first_char) {
	case 't':
		sht15_read();
		break;
	case '2':
		
		break;
	case '3':
		
		break;
	case 'a':
		
		break;
	case 'b':
		break;
		
	case 'c':
		break;
	case 'd':
		break;
	case 'u':
		break;
	case 'p':
		break;
	default:
		break;
	} // switch		

}

	
// configure_system
//
// Get the pic ready for everything we want to do

void configure_system() {
	    
	kill_interrupts();	// turn off interrupts just in case

	turn_analog_inputs_off();	// kill those pesky analogue inputs

	platform_leds_setup_io();


	// real time clock setup ----------------------------


	sht15_setup_io();
	
	serial_setup(SPBRG_115200);

	term_init();	// initialise terminal 

	// LED setup ----------------------------
	
	platform_leds_setup_io();
	
	// general setup ----------------------------

	// Timer 0 is used for ticks - 1ms at 40mhz (10mips)
	timer_setup_0(TIMER_16BIT_MODE, TIMER_PRESCALER_OFF, 0xffff - 10000 -1); 

	turn_peripheral_ints_on();
	turn_global_ints_on();

}




void main() {

uns8 test_result, test_count;
uns16 marker_tick, test_tick;

	configure_system();
	delay_ms(100);

	serial_print_str("\n\nPicPack MOD-1005 demo\n");

	serial_print_str( "\n<");
	serial_print_str(__TIME__);
	serial_putc(' ');
	serial_print_str(__DATE__);
	serial_print_str(">\n");
	
	timer_start_0();

	
	test_count = 0;
	marker_tick = tick_get_count();	// find out what we're up to

	for(;;) {	// for ever
		
		term_process();	// process any typing on the terminal
			
		
		test_tick = tick_get_count();	// find out what we're up to

		if (tick_calc_diff(marker_tick, test_tick) >= 1000) {	// 1000 - it's a second, 2000 = 2 seconds
			//serial_print_str("Test\n");
			marker_tick = test_tick;
		}
		
		start_crit_sec();
			platform_leds_process();
		end_crit_sec();


		

	}
	
}	// main

