//-----------------------------------------------------
// Pic Pack library
// 
// its_m2_temp.c
//
//
// (c) Ian Harris 2009
// imharris [at] gmail.com
//
// 
//-----------------------------------------------------


// Pic Pack includes
#include "config.h"
#include "pic_utils.h"
#include "pic_serial.h"
#include "mrf24j40.h"
#include "spi_hw.h"
#include "wpan.h"
#include "pic_timer.h"
#include "pic_tick.h"

#include "its_mode2.h"
#include "its_common.h"

// System includes
#include "string.h"
#include "stdlib.h"

#include "i2c.h"
#include "ds1631.h"
#define DS1631_ADDR 0x00

bit temp_requested = 0;
uns16 request_id;

// Put received characters in here to see
// if they make a command
char serial_buffer[6]= '\0';

// See if we need to do a task
uns8 do_task = 0;

// Flag if we received something
bit receive = 0;

// Dummy variable to show we can set it 
uns8 send_to = 0;


bit promiscuous_mode = 0;

// Interrupt routine - - - - - - - - - -

void interrupt() {
	
	if (test_bit(intcon, INT0IF)) {
		clear_bit(intcon, INT0IF);
		mrf24j40_handle_isr();
	}	
	serial_handle_tx_isr();
	serial_handle_rx_isr();
	timer_handle_0_isr();	
}


	


// Process_rx
//
// Keep adding characters to our input buffer
// when we enter a command, then act on it

void process_rx() {
	
	uns8 len,	// length of string
		 rec;	// received character
	uns8 first_char;
	
	rec = serial_getc();	// get the character from the fifo
	
	serial_putc(rec);	// print it out so we can see what we typed
	
	if (rec == MAGIC_BOOSTBLOADER_REQUEST) {
		boostbloader();
	}

	if (rec == '\r') {	// did we press return?
		first_char = serial_buffer[0];
		switch (first_char) {
			case '1':
				do_task = 1;
				break;
		    case '2':
				do_task = 2;
				break;
		    case '3':
				do_task = 9;
				break;
			case 'c':
				do_task = 3;
				break;
			case 'd':
				do_task = 4;
				break;
			case 'e':
				do_task = 8;
				break;
			case 'p':
				do_task = 5;
				break;	
			case 'r':
				do_task = 6;
				break;								
			case 't':
				do_task = 7;
				break;
			case 'q':
				do_task = 10;
				break;
			case 'h':
				do_task = 11;
			default:
				// More complicated example
				// Let's set a variable based on input
				// You can type:
				// s05
				// And press enter, it will set "send_to" to this (in hex)
				if ((first_char == 's') && (strlen(serial_buffer)==3)) {
					send_to = atoui_hex(&serial_buffer[1]);
					serial_print_str(" Set send_to to ");
					serial_print_int(send_to);	// Of course, this could be anything
					serial_putc('\n');
				};
		}		
		serial_buffer[0] = '\0';	// clear the buffer
        serial_putc('\n');	// print new line
        serial_putc('>');	// print the prompt
	} else {
		len = strlen(serial_buffer);	// add the character to our line input
		serial_buffer[len] = rec;
		serial_buffer[len+1] = '\0';
	}

}

// configure_system
//
// Get the pic ready for everything we want to do

void configure_system() {
	    
	kill_interrupts();	// turn off interrupts just in case

	turn_analog_inputs_off();	// kill those pesky analogue inputs

	clear_bit(intcon, INT0IF);	// in case it was triggered already
	clear_bit(intcon2, INTEDG0);	// Need falling edge for mrf24j40 defaults
	set_bit(intcon, INT0IE);	// turn on rb0/int0 interrupts
	
	spi_hw_setup_io();
	spi_hw_init();
	mrf24j40_setup_io();
	mrf24j40_init();

	#ifdef _PIC16
		timer_setup_0(TIMER_8BIT_MODE, TIMER_PRESCALER_1_TO_8, 0xff - 250 - 1);	// 1ms at 8Mhz
	#endif

	#ifdef _PIC18
		timer_setup_0(TIMER_16BIT_MODE, TIMER_PRESCALER_OFF, 0xffff - 5000);	// 1ms at 20Mhz
		//timer_setup_0(TIMER_16BIT_MODE, TIMER_PRESCALER_1_TO_2, 0xffff - 1500 + 7);	// 1ms at 12Mhz
		//timer_setup_0(TIMER_16BIT_MODE, TIMER_PRESCALER_1_TO_8, 0xffff - 375 + 1);	// 1ms at 12Mhz
	#endif	

	ds1631_setup_io();
	// 12 bit temp, one shot mode
	ds1631_set_config(DS1631_ADDR, 0b00001101);

		
	serial_setup(SPBRG_115200);

	turn_peripheral_ints_on();
	turn_global_ints_on();
}


// Handle any command line tasks we need to do
// Very handy to print out variables, do commands,
// print out some debug, etc...

void handle_tasks() {
	
	uns8 data[10];
	uns8 EA_1[8] = { 0, 0, 0, 0, 0, 0, 0, 1 };
	uns8 EA_2[8] = { 0, 0, 0, 0, 0, 0, 0, 2 };
	uns8 EA_3[8] = { 0, 0, 0, 0, 0, 0, 0, 3 };
	
	switch (do_task) {
		case 1:
			serial_print_str("TX1... ");
			data[0] = 'T';
			data[1] = 'O';
			// we just happen to know the device is 0x2020
			its2_router_queue_packet(0x0001, ITS_GENERIC_DATA, &data, 2, 1);

			break;
		case 2:
			serial_print_str("TX2.... ");
			data[0] = 'o';
			data[1] = 't';
			its2_router_queue_packet(0x0002, ITS_GENERIC_DATA, &data, 2, 1);
			
			break;
		case 9:
			serial_print_str("TX3.... ");
			data[0] = 'T';
			data[1] = 't';
			its2_router_queue_packet(0x0003, ITS_GENERIC_DATA, &data, 2, 1);
			
			break;
		case 3:
			serial_print_str("Setting address to 1\n");
			mrf24j40_set_extended_address(&EA_1);
			its2_router_init(0x0001, ITS_NETWORK_ID);
			
			break;
		case 4:
			serial_print_str("Setting address to 2\n");
			mrf24j40_set_extended_address(&EA_2);
			its2_router_init(0x0002, ITS_NETWORK_ID);
			break;
		case 8:
			serial_print_str("Setting address to 3\n");
			mrf24j40_set_extended_address(&EA_3);
			its2_router_init(0x0003, ITS_NETWORK_ID);
			break;
		case 5:
			serial_print_str("Setting pan id = 5\n");
			mrf24j40_set_pan_id(0x05);
			break;
		case 6:
			if (promiscuous_mode) {
				serial_print_str("Prom OFF\n");
				promiscuous_mode = 0;
				mrf24j40_short_addr_write(RXMCR, 0x00); 
			} else {
				promiscuous_mode = 1;
				serial_print_str("Prom ON\n");
				mrf24j40_short_addr_write(RXMCR, 0x01); // promiscuous mode, no coord
			}
			break;
		case 10:
				serial_print_str("debug mode\n");
				debug_module = 1;
				break;
		case 11:
				its_print_devices();
				break;
			
		case 7:
			its2_print_queue();
			/*mrf24j40_long_addr_write(0x80, 0xff);
			uns8 val = mrf24j40_long_addr_read(0x80);
			serial_print_str("Read from 0x080 = ");
			serial_print_int_hex(val);
			serial_print_nl(); */
			/*uns8 lowest_channel = mrf24j40_scan_for_lowest_channel_ed();

			serial_print_str("Lowest channel = ");
			serial_print_int(lowest_channel);
			serial_print_nl();	*/	
			break;
	} 	// switch		

	do_task = 0;
}

void its2_router_receive_callback(uns16 device_id, uns8 *data, uns8 data_length) {
		serial_print_str("Controller receives from ");
		serial_print_int(device_id);
		serial_print_nl();
		request_id = device_id;
		temp_requested = 1;
		ds1631_convert_temp(DS1631_ADDR);	
}

void main() {

	uns8 my_ea[8] = { 0, 0, 0, 0, 0, 0, 0, 3};
	
	configure_system();
	
	delay_ms(100);
	delay_ms(100);
	
	serial_print_str("\n\nITS M2 Temperature Sensor\n");
	serial_print_str( "\n<");
	serial_print_str(__TIME__);
	serial_putc(' ');
	serial_print_str(__DATE__);
	serial_print_str(">\n");

	serial_print_str("Commands: a,x,c,n,s00\n");

//	its2_router_init(/* device */ 0x1010, /* network id */ 0x5000);
	serial_print_str("Setting address to 3\n");
	mrf24j40_set_extended_address(&my_ea);
	its2_router_init(0x0003, ITS_NETWORK_ID);
	
	timer_start_0();
	
	uns16 temp;
	
	for(;;) {

		if (serial_rx_avail()) {
			process_rx();
		}

		wpan_process();	
		
		its2_router_process();
		
		handle_tasks();	  

		if (temp_requested) {	// temp requested, so lets see if it has finished
			uns8 config = ds1631_get_config(DS1631_ADDR);
			if (test_bit(config, 7)) {	// the DONE bit
				temp_requested = 0;
				temp = ds1631_get_temp(DS1631_ADDR);
				serial_print_str("Temp = ");
				serial_print_int_hex_16bit(temp);
				serial_print_nl();
				its2_router_queue_packet(request_id, ITS_GENERIC_DATA, (uns8*)&temp, 2, ITS2_FLAG_ACK);

			}	
		}


	}	// while (1)

	
}	// main

