//-----------------------------------------------------
// Pic Pack library
// 
// platform_flasher_demo.c
//
// Demonstration routines for the platform_leds code.
//
// Ian Harris 2010
// www.embeddedadventures.com
//
//-----------------------------------------------------


// Pic Pack includes
#include "config.h"

// General PIC utilities
#include "pic_utils.h"
// Timer 0 functions
#include "pic_timer.h"
// Tick functions (builds on timer functions)
#include "pic_tick.h"
// Serial port communications
#include "pic_serial.h"
// Terminal functions (builds on serial port)
#include "pic_term.h"
// Platform LEDs (what this demo is all about)
#include "platform_leds.h"

#include "config.h"

// Dummy variable to show we can set it 

uns8 my_var = 0;

// Interrupt routine - - - - - - - - - -

void interrupt() {
		// Handle timer 0 interrupt
		timer_handle_0_isr();	
		
		// Handle serial interrupts
		serial_handle_tx_isr();
		serial_handle_rx_isr();
}

//
// term_entry_callback will be called when 
// user pressed <enter> on serial "terminal".
// term_buffer contains the characters (if any)
// that the user has entered.

void term_entry_callback(uns8 *term_buffer) {

uns8 data[5];
uns8 first_char;
	
	first_char = term_buffer[0];

	switch (first_char) {
		case '1':
			serial_print_str("Flash LED 1\n");
			platform_leds_flash(1);
			break;
		case '2':
			serial_print_str("Flash LED 2\n");
			platform_leds_flash(1);
			break;
		case '3':
			serial_print_str("Flash LED 3\n");
			platform_leds_flash(3);
			break;
		case 'c':
			serial_print_str("Continuously flash on led ");
			switch (term_buffer[1]) {
				case '1': 
					serial_putc('1');
					platform_leds_flashing(1, 1);
					break;
				case '2': 
					serial_putc('2');
					platform_leds_flashing(2, 1);
					break;
				case '3': 
					serial_putc('3');
					platform_leds_flashing(3, 1);
					break;
			}		
					
			break;
		case 'n':
			serial_print_str("Single flash on led ");
			switch (term_buffer[1]) {
				case '1': 
					serial_putc('1');
					platform_leds_flashing(1, 0);
					break;
				case '2': 
					serial_putc('2');
					platform_leds_flashing(2, 0);
					break;
				case '3': 
					serial_putc('3');
					platform_leds_flashing(3, 0);
					break;
			}		
					
			break;
		default:
			serial_print_str("Unknown command.\n");
			break;
	}	// switch		

}	// term_entry_callback


// configure_system
//
// Get the pic ready for everything we want to do

void configure_system() {
	
	// turn off interrupts just in case
	kill_interrupts();
	
	// kill those pesky analogue inputs
	turn_analog_inputs_off();	
	
	// set serial to run at 9600bps 
	serial_setup(SPBRG_9600);
	
	// initialise termainl
	term_init();

	// setup i/o for platform LEDs
	platform_leds_setup_io();

	// Timer 0 is used for ticks - 1ms at 40mhz (10mips)
	timer_setup_0(TIMER_16BIT_MODE, TIMER_PRESCALER_OFF, 0xffff - 10000 -1); 

	// tern peripheral interrupts on (ie, serial port)
	turn_peripheral_ints_on();
	
	// turn all interrupts on
	turn_global_ints_on();
}



void main() {
	
	configure_system();
	
	delay_ms(100);	// mostly so screamer bootloader has time to switch windows
	
	timer_start_0();
	
	serial_print_str("\n\nPlatform LEDs\n");
	serial_print_str( "Press 1, 2, 3 to flash your LED\nc1, c2, c3 to continuously flash\nn1,n2,n3 to single flash\n<");
	
	// Print the time and date this file was compiled. Handy to know which
	// version you are running.
	
	serial_print_str(__TIME__);
	serial_putc(' ');
	serial_print_str(__DATE__);
	serial_print_str(">\n");

	serial_print_str("Flashing LED 1 continuously.\n");
	platform_leds_flashing(1, 1);
	platform_leds_flash(1);
	
	for(;;) {

		term_process();
		platform_leds_process();

	}

	
}	// main

