/*

Copyright (c) 2010, Embedded Adventures, www.embeddedadventures.com
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

- Neither the name of Embedded Adventures nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
THE POSSIBILITY OF SUCH DAMAGE.

Contact us at admin@embeddedadventures.com
*/

/** \file drv_ea_ldp6416.c
    \brief Draw drivers for Embedded Adventures LDP-6416 LED panel and similar


*/



#include "ea_ldp6416.h"
#include "config.h"
#include "draw\draw.h"
#include "draw\draw_screen_buffer.h"

#define set_pins_r1_g1() \
	set_pin(ea_ldp6416_r1_port, ea_ldp6416_r1_pin); \
	set_pin(ea_ldp6416_g1_port, ea_ldp6416_g1_pin); 

uns8 current_row =0;

uns8 bright_count = 0;
uns8 bright_level = 0;
uns8 buffer_position = 0;
#if ea_ldp6416_displays > 1
	uns8 buffer_position1 = 0;
#endif	
uns8 current_buffer = 0;

char buffer0[256];
#if ea_ldp6416_displays > 1
	char buffer1[256];
#endif
#if ea_ldp6416_displays > 2
	char buffer2[256];
#endif	
#if ea_ldp6416_displays > 3
	char buffer3[256];
#endif
	
void drv_paint() {
	uns8 count;
	// start_crit_sec();
	count = 0;
	
	do {
		buffer0[count] = draw_buffer0[count];
		#if ea_ldp6416_displays > 1
			buffer1[count] = draw_buffer1[count];
		#endif;
		#if ea_ldp6416_displays > 2
			buffer2[count] = draw_buffer2[count];
		#endif
		#if ea_ldp6416_displays > 3
			buffer3[count] = draw_buffer3[count];
		#endif
		count++;
	} while (count !=0);
	//end_crit_sec();	
}

void drv_print_buffer() {
uns8 count;

	count = 0;
	do {
		serial_print_int(buffer0[count]);
		serial_print_spc();
		count++;
	} while (count != 0);	
	serial_print_nl();

}


void drv_set_display_brightness(uns8 brightness) {
	if (brightness < ea_ldp6416_max_brightness) {
		bright_level = brightness;
	} else {
		bright_level = ea_ldp6416_max_brightness;
	}
}	

void drv_refresh() {


uns8 x;
uns8 data;

	
#if ea_ldp6416_max_brightness > 0
	bright_count++;
	
	if (bright_count > bright_level) {
		// Turn off display
		set_pin(ea_ldp6416_en_port, ea_ldp6416_en_pin);
	}
	
	if (bright_count == ea_ldp6416_max_brightness) {
		bright_count = 255;
#endif
				
		for (x = 0; x < x_refresh; x++) {	// WAS 16
			#if ea_ldp6416_displays == 1
				data = buffer0[buffer_position];
			#endif

			#if ea_ldp6416_displays == 2 && ea_ldp6416_display_orientation == HORIZONTAL
				if (current_buffer == 0) {
					data = buffer0[buffer_position];
				} else {
					data = buffer1[buffer_position];
				}
			#endif	
			#if ea_ldp6416_displays == 3 && ea_ldp6416_display_orientation == HORIZONTAL
				switch (current_buffer) {
					case 0: data = buffer0[buffer_position]; break;
					case 1: data = buffer1[buffer_position]; break;
					case 2: data = buffer2[buffer_position]; break;
				} 
			#endif
			#if ea_ldp6416_displays == 4 && ea_ldp6416_display_orientation == HORIZONTAL
				switch (current_buffer) {
					case 0: data = buffer0[buffer_position]; break;
					case 1: data = buffer1[buffer_position]; break;
					case 2: data = buffer2[buffer_position]; break;
					case 3: data = buffer3[buffer_position]; break;
				} 
			#endif
			
			set_pins_r1_g1();
			
			if (data.0) { clear_pin(ea_ldp6416_r1_port, ea_ldp6416_r1_pin); }
			if (data.1) { clear_pin(ea_ldp6416_g1_port, ea_ldp6416_g1_pin); }
			
			clear_pin(ea_ldp6416_s_port, ea_ldp6416_s_pin);
			set_pin  (ea_ldp6416_s_port, ea_ldp6416_s_pin);
			
			set_pins_r1_g1();
			
			if (data.2) { clear_pin(ea_ldp6416_r1_port, ea_ldp6416_r1_pin); }
			if (data.3) { clear_pin(ea_ldp6416_g1_port, ea_ldp6416_g1_pin); }
			
			clear_pin(ea_ldp6416_s_port, ea_ldp6416_s_pin);
			set_pin  (ea_ldp6416_s_port, ea_ldp6416_s_pin);
			
			set_pins_r1_g1();
			
			if (data.4) { clear_pin(ea_ldp6416_r1_port, ea_ldp6416_r1_pin); }
			if (data.5) { clear_pin(ea_ldp6416_g1_port, ea_ldp6416_g1_pin); }
				
			clear_pin(ea_ldp6416_s_port, ea_ldp6416_s_pin);
			set_pin  (ea_ldp6416_s_port, ea_ldp6416_s_pin);
			
			set_pins_r1_g1();
			
			if (data.6) { clear_pin(ea_ldp6416_r1_port, ea_ldp6416_r1_pin); }
			if (data.7) { clear_pin(ea_ldp6416_g1_port, ea_ldp6416_g1_pin); }
				
			clear_pin(ea_ldp6416_s_port, ea_ldp6416_s_pin);
			set_pin  (ea_ldp6416_s_port, ea_ldp6416_s_pin);
			
			buffer_position++;
			
			#if ea_ldp6416_displays > 1 && ea_ldp6416_display_orientation == HORIZONTAL
				if (buffer_position == 0) {
					current_buffer++;
				}	
			#endif	
		} 	
	
		#if ea_ldp6416_displays == 2 && ea_ldp6416_display_orientation == VERTICAL
	
			for (x = 0; x < x_refresh; x++) {
				data = buffer1[buffer_position1];
				
				set_pins_r1_g1();
				
				if (data_upper.0) { clear_pin(ea_ldp6416_r1_port, ea_ldp6416_r1_pin); }
				if (data_upper.1) { clear_pin(ea_ldp6416_g1_port, ea_ldp6416_g1_pin); }
					
				clear_pin(ea_ldp6416_s_port, ea_ldp6416_s_pin);
				set_pin  (ea_ldp6416_s_port, ea_ldp6416_s_pin);
				
				set_pins_r1_g1();
				
				if (data_upper.2) { clear_pin(ea_ldp6416_r1_port, ea_ldp6416_r1_pin); }
				if (data_upper.3) { clear_pin(ea_ldp6416_g1_port, ea_ldp6416_g1_pin); }
				
				clear_pin(ea_ldp6416_s_port, ea_ldp6416_s_pin);
				set_pin  (ea_ldp6416_s_port, ea_ldp6416_s_pin);
				
				set_pins_r1_g1();
				
				if (data_upper.4) { clear_pin(ea_ldp6416_r1_port, ea_ldp6416_r1_pin); }
				if (data_upper.5) { clear_pin(ea_ldp6416_g1_port, ea_ldp6416_g1_pin); }
				
				clear_pin(ea_ldp6416_s_port, ea_ldp6416_s_pin);
				set_pin  (ea_ldp6416_s_port, ea_ldp6416_s_pin);
				
				set_pins_r1_g1();
				
				if (data_upper.6) { clear_pin(ea_ldp6416_r1_port, ea_ldp6416_r1_pin); }
				if (data_upper.7) { clear_pin(ea_ldp6416_g1_port, ea_ldp6416_g1_pin); }
					
				clear_pin(ea_ldp6416_s_port, ea_ldp6416_s_pin);
				set_pin  (ea_ldp6416_s_port, ea_ldp6416_s_pin);
				
				buffer_position1++;
			} // x loop	
			
		#endif
			
	
		set_pin(ea_ldp6416_en_port, ea_ldp6416_en_pin);	// turn enable off
		
		clear_pin(ea_ldp6416_a_port, ea_ldp6416_a_pin);
		clear_pin(ea_ldp6416_b_port, ea_ldp6416_b_pin);
		clear_pin(ea_ldp6416_c_port, ea_ldp6416_c_pin);
		clear_pin(ea_ldp6416_d_port, ea_ldp6416_d_pin);
		
		if (current_row.0) { set_pin(ea_ldp6416_a_port, ea_ldp6416_a_pin); }
		if (current_row.1) { set_pin(ea_ldp6416_b_port, ea_ldp6416_b_pin); }	
		if (current_row.2) { set_pin(ea_ldp6416_c_port, ea_ldp6416_c_pin); }		
		if (current_row.3) { set_pin(ea_ldp6416_d_port, ea_ldp6416_d_pin); } 

		// latch data	
		set_pin(ea_ldp6416_l_port, ea_ldp6416_l_pin);
		clear_pin(ea_ldp6416_l_port, ea_ldp6416_l_pin);
		
		// enable display of row
		clear_pin(ea_ldp6416_en_port, ea_ldp6416_en_pin);
	
		current_row++;
		if (current_row == y_refresh) {
			current_buffer = 0;
			buffer_position = 0;
			current_row = 0;
		}	

#if ea_ldp6416_max_brightness > 0

	}
#endif

}
uns8 drv_get_pixel(uns8 x, uns8 y) {
	//Should do something to look up the pixel
}

void drv_setup_io() {
	ea_ldp6416_setup_io();
}

void drv_init() {
	ea_ldp6416_init();
}	
