/*

Copyright (c) 2010, Embedded Adventures, www.embeddedadventures.com
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

- Neither the name of Embedded Adventures nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
THE POSSIBILITY OF SUCH DAMAGE.

Contact us at admin@embeddedadventures.com
*/

/** \file drv_ea_ldp6432.c
    \brief Draw drivers for Embedded Adventures LDP-6432 LED panel and similar


*/

#ifndef __draw_drv_ea_ldp6432_h
#define __draw_drv_ea_ldp6432_h

#include "ea_ldp6432.h"
#include "config.h"
#include "draw\draw.h"
#include "draw\draw_screen_buffer.h"

#define set_pins_r1_g1_r2_g2() \
	set_pin(ea_ldp6432_r1_port, ea_ldp6432_r1_pin); \
	set_pin(ea_ldp6432_r2_port, ea_ldp6432_r2_pin); \
	set_pin(ea_ldp6432_g1_port, ea_ldp6432_g1_pin); \
	set_pin(ea_ldp6432_g2_port, ea_ldp6432_g2_pin)  

char buffer0[256];
char buffer1[256];
#if ea_ldp6432_displays == 2
	char buffer2[256];
	char buffer3[256];
#endif

uns8 current_row =0;

uns8 bright_count = 0;
uns8 bright_level = 3;	// brightness = 0(dim),1,2,3(bright)
uns8 buffer_position0 = 0;
#if ea_ldp6432_displays == 2
	uns8 buffer_position1 = 0;
#endif

#define MAX_BRIGHTNESS 3

	
void drv_paint() {
	uns8 count;
	
	count = 0;
	
	do {
		buffer0[count] = draw_buffer0[count];
		buffer1[count] = draw_buffer1[count];
		#if ea_ldp6432_displays == 2
			buffer2[count] = draw_buffer2[count];
			buffer3[count] = draw_buffer3[count];
		#endif
		count++;
	} while (count !=0);	
}

void drv_set_display_brightness(uns8 brightness) {
	if (brightness < MAX_BRIGHTNESS) {
		bright_level = brightness;
	} else {
		bright_level = MAX_BRIGHTNESS;
	}
}	

void drv_refresh() {


uns8 x;
uns8 data_upper, data_lower;


	bright_count++;
	
	if (bright_count > bright_level) {
		// Turn off display
		set_pin(ea_ldp6432_en_port, ea_ldp6432_en_pin);
	}
	
	if (bright_count == MAX_BRIGHTNESS) {
		bright_count = 255;

	
			
		for (x = 0; x < 16; x++) {
			data_upper = buffer0[buffer_position0];
			data_lower = buffer1[buffer_position0];
			
			set_pins_r1_g1_r2_g2();
			
			if (data_upper.0) { 
				clear_pin(ea_ldp6432_r1_port, ea_ldp6432_r1_pin);
			}
			if (data_upper.1) {
				clear_pin(ea_ldp6432_g1_port, ea_ldp6432_g1_pin);
			}
			if (data_lower.0) {
				clear_pin(ea_ldp6432_r2_port, ea_ldp6432_r2_pin);
			}
			if (data_lower.1) {
				clear_pin(ea_ldp6432_g2_port, ea_ldp6432_g2_pin);
			}
				
			clear_pin(ea_ldp6432_s_port, ea_ldp6432_s_pin);
			set_pin  (ea_ldp6432_s_port, ea_ldp6432_s_pin);
			
			set_pins_r1_g1_r2_g2();
			
			if (data_upper.2) { 
				clear_pin(ea_ldp6432_r1_port, ea_ldp6432_r1_pin);
			}
			if (data_upper.3) {
				clear_pin(ea_ldp6432_g1_port, ea_ldp6432_g1_pin);
			}
			if (data_lower.2) {
				clear_pin(ea_ldp6432_r2_port, ea_ldp6432_r2_pin);
			}
			if (data_lower.3) {
				clear_pin(ea_ldp6432_g2_port, ea_ldp6432_g2_pin);
			}
				
			clear_pin(ea_ldp6432_s_port, ea_ldp6432_s_pin);
			set_pin  (ea_ldp6432_s_port, ea_ldp6432_s_pin);
			
			set_pins_r1_g1_r2_g2();
			
			if (data_upper.4) { 
				clear_pin(ea_ldp6432_r1_port, ea_ldp6432_r1_pin);
			}
			if (data_upper.5) {
				clear_pin(ea_ldp6432_g1_port, ea_ldp6432_g1_pin);
			}
			if (data_lower.4) {
				clear_pin(ea_ldp6432_r2_port, ea_ldp6432_r2_pin);
			}
			if (data_lower.5) {
				clear_pin(ea_ldp6432_g2_port, ea_ldp6432_g2_pin);
			}
				
			clear_pin(ea_ldp6432_s_port, ea_ldp6432_s_pin);
			set_pin  (ea_ldp6432_s_port, ea_ldp6432_s_pin);
			
			set_pins_r1_g1_r2_g2();
			
			
			if (data_upper.6) { 
				clear_pin(ea_ldp6432_r1_port, ea_ldp6432_r1_pin);
			}
			if (data_upper.7) {
				clear_pin(ea_ldp6432_g1_port, ea_ldp6432_g1_pin);
			}
			if (data_lower.6) {
				clear_pin(ea_ldp6432_r2_port, ea_ldp6432_r2_pin);
			}
			if (data_lower.7) {
				clear_pin(ea_ldp6432_g2_port, ea_ldp6432_g2_pin);
			}
				
			clear_pin(ea_ldp6432_s_port, ea_ldp6432_s_pin);
			set_pin  (ea_ldp6432_s_port, ea_ldp6432_s_pin);
			
			buffer_position0++;
		} 	
		
			#if ea_ldp6432_displays == 2
				for (x = 0; x < 16; x++) {
					data_upper = buffer2[buffer_position1];
					data_lower = buffer3[buffer_position1];
					
					set_pins_r1_g1_r2_g2();
					
					if (data_upper.0) { 
						clear_pin(ea_ldp6432_r1_port, ea_ldp6432_r1_pin);
					}
					if (data_upper.1) {
						clear_pin(ea_ldp6432_g1_port, ea_ldp6432_g1_pin);
					}
					if (data_lower.0) {
						clear_pin(ea_ldp6432_r2_port, ea_ldp6432_r2_pin);
					}
					if (data_lower.1) {
						clear_pin(ea_ldp6432_g2_port, ea_ldp6432_g2_pin);
					}
						
					clear_pin(ea_ldp6432_s_port, ea_ldp6432_s_pin);
					set_pin  (ea_ldp6432_s_port, ea_ldp6432_s_pin);
					
					set_pins_r1_g1_r2_g2();
					
					if (data_upper.2) { 
						clear_pin(ea_ldp6432_r1_port, ea_ldp6432_r1_pin);
					}
					if (data_upper.3) {
						clear_pin(ea_ldp6432_g1_port, ea_ldp6432_g1_pin);
					}
					if (data_lower.2) {
						clear_pin(ea_ldp6432_r2_port, ea_ldp6432_r2_pin);
					}
					if (data_lower.3) {
						clear_pin(ea_ldp6432_g2_port, ea_ldp6432_g2_pin);
					}
						
					clear_pin(ea_ldp6432_s_port, ea_ldp6432_s_pin);
					set_pin  (ea_ldp6432_s_port, ea_ldp6432_s_pin);
					
					set_pins_r1_g1_r2_g2();
					
					if (data_upper.4) { 
						clear_pin(ea_ldp6432_r1_port, ea_ldp6432_r1_pin);
					}
					if (data_upper.5) {
						clear_pin(ea_ldp6432_g1_port, ea_ldp6432_g1_pin);
					}
					if (data_lower.4) {
						clear_pin(ea_ldp6432_r2_port, ea_ldp6432_r2_pin);
					}
					if (data_lower.5) {
						clear_pin(ea_ldp6432_g2_port, ea_ldp6432_g2_pin);
					}
						
					clear_pin(ea_ldp6432_s_port, ea_ldp6432_s_pin);
					set_pin  (ea_ldp6432_s_port, ea_ldp6432_s_pin);
					
					set_pins_r1_g1_r2_g2();
					
					
					if (data_upper.6) { 
						clear_pin(ea_ldp6432_r1_port, ea_ldp6432_r1_pin);
					}
					if (data_upper.7) {
						clear_pin(ea_ldp6432_g1_port, ea_ldp6432_g1_pin);
					}
					if (data_lower.6) {
						clear_pin(ea_ldp6432_r2_port, ea_ldp6432_r2_pin);
					}
					if (data_lower.7) {
						clear_pin(ea_ldp6432_g2_port, ea_ldp6432_g2_pin);
					}
						
					clear_pin(ea_ldp6432_s_port, ea_ldp6432_s_pin);
					set_pin  (ea_ldp6432_s_port, ea_ldp6432_s_pin);
					
					buffer_position1++;
				} // x loop	
				
			#endif
					
	
		set_pin(ea_ldp6432_en_port, ea_ldp6432_en_pin);	// turn enable off
		
		clear_pin(ea_ldp6432_a_port, ea_ldp6432_a_pin);
		clear_pin(ea_ldp6432_b_port, ea_ldp6432_b_pin);
		clear_pin(ea_ldp6432_c_port, ea_ldp6432_c_pin);
		clear_pin(ea_ldp6432_d_port, ea_ldp6432_d_pin);
		if (current_row.0) { set_pin(ea_ldp6432_a_port, ea_ldp6432_a_pin); }		
		if (current_row.1) { set_pin(ea_ldp6432_b_port, ea_ldp6432_b_pin); }	
		if (current_row.2) { set_pin(ea_ldp6432_c_port, ea_ldp6432_c_pin); }		
		if (current_row.3) { set_pin(ea_ldp6432_d_port, ea_ldp6432_d_pin); } 
	
		// latch data	
		set_pin(ea_ldp6432_l_port, ea_ldp6432_l_pin);
		clear_pin(ea_ldp6432_l_port, ea_ldp6432_l_pin);
		
		// enable display of line
		clear_pin(ea_ldp6432_en_port, ea_ldp6432_en_pin);
		
		current_row++;
			
		if (current_row == 16) {
			buffer_position0 = 0;
			#if ea_ldp6432_displays == 2
				buffer_position1 = 0;
			#endif			
			current_row = 0;
		}	

	}
}

void drv_print_buffer() {
uns8 count;

	count = 0;
	do {
		serial_print_int(buffer0[count]);
		serial_print_spc();
		count++;
	} while (count != 0);	
	serial_print_nl();

}



uns8 drv_get_pixel(uns8 x, uns8 y) {
	//ht1632_get_pixel(x, y);
}

void drv_clear_screen() {
	//ht1632_clear_screen();
}

void drv_setup_io() {
	ea_ldp6432_setup_io();
}

void drv_init() {
	ea_ldp6432_init();
}	

#endif
