/*

Copyright (c) 2010, Embedded Adventures, www.embeddedadventures.com
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

- Neither the name of Embedded Adventures nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
THE POSSIBILITY OF SUCH DAMAGE.

Contact us at admin@embeddedadventures.com
*/

/** \file drv_ea_ldp8008.c
    \brief Draw drivers for Embedded Adventures LDP-8008 LED panel and similar


*/
#ifndef __draw_drv_ea_ldp8008_h
#define __draw_drv_ea_ldp8008_h

#include "ea_ldp8008.h"
#include "config.h"
#include "draw\draw.h"
#include "draw\draw_screen_buffer.h"

#define set_pins_r_g() \
	set_pin(ea_ldp8008_r_port, ea_ldp8008_r_pin); \
	set_pin(ea_ldp8008_g_port, ea_ldp8008_g_pin); 

uns8 current_row =0;

uns8 bright_count = 0;
uns8 bright_level = 3;
uns8 buffer_position0 = 0;
#define MAX_BRIGHTNESS 3



char buffer0[256];
#if ea_ldp8008_displays > 1
char buffer1[256];
#endif
#if ea_ldp8008_displays > 2
	char buffer2[256];
#endif	
#if ea_ldp8008_displays > 3
	char buffer3[256];
#endif
	
void drv_paint() {
	uns8 count;
	start_crit_sec();
	count = 0;
	
	do {
		buffer0[count] = draw_buffer0[count];
		#if ea_ldp8008_displays == 2
			buffer1[count] = draw_buffer1[count];
		#endif;
		#if ea_ldp8008_displays == 3
			buffer2[count] = draw_buffer2[count];
		#endif
		#if ea_ldp8008_displays == 4
			buffer3[count] = draw_buffer3[count];
		#endif
		count++;
	} while (count !=160);
	end_crit_sec();	
}

void drv_print_buffer() {
uns8 count;

	count = 0;
	do {
		serial_print_int(buffer0[count]);
		serial_print_spc();
		count++;
	} while (count != 0);	
	serial_print_nl();

}


void drv_set_display_brightness(uns8 brightness) {
	if (brightness < MAX_BRIGHTNESS) {
		bright_level = brightness;
	} else {
		bright_level = MAX_BRIGHTNESS;
	}
}	

void drv_refresh() {


uns8 x;
uns8 data;

	bright_count++;

	if (bright_count > bright_level) {
		// Turn off display
		clear_pin(ea_ldp8008_en_port, ea_ldp8008_en_pin);
	}
	//if (bright_count == 1) {
	//	bright_count = 0;
	//}	
	if (bright_count == MAX_BRIGHTNESS) {
		bright_count = 255;
	

	// Start upper at the top

	//	count_lower = 64 * 2 * 16 / 8;	// Start lower at the second half
	#if ea_ldp8008_displays == 2
	#warning "This doesn't work!!"
	//		uns8 count_byte_disp2 = 0;
		//uns16 count_upper_disp2, count_lower_disp2;
	//	count_upper_disp2 = (64 * 2 * 16 / 8) * 2;
	//	count_lower_disp2 = (64 * 2 * 16 / 8) * 3;
	#endif
		
	for (x = 0; x < 20; x++) {	// 20*8bits/2bpp = 80 pixels across
		data = buffer0[buffer_position0];
		
		set_pins_r_g();
		
		if (data.0) { 
			clear_pin(ea_ldp8008_r_port, ea_ldp8008_r_pin);
		}
		if (data.1) {
			clear_pin(ea_ldp8008_g_port, ea_ldp8008_g_pin);
		}
			
		clear_pin(ea_ldp8008_s_port, ea_ldp8008_s_pin);
		set_pin  (ea_ldp8008_s_port, ea_ldp8008_s_pin);
		
		set_pins_r_g();
		
		if (data.2) { 
			clear_pin(ea_ldp8008_r_port, ea_ldp8008_r_pin);
		}
		if (data.3) {
			clear_pin(ea_ldp8008_g_port, ea_ldp8008_g_pin);
		}
		
		clear_pin(ea_ldp8008_s_port, ea_ldp8008_s_pin);
		set_pin  (ea_ldp8008_s_port, ea_ldp8008_s_pin);
		
		set_pins_r_g();
		
		if (data.4) { 
			clear_pin(ea_ldp8008_r_port, ea_ldp8008_r_pin);
		}
		if (data.5) {
			clear_pin(ea_ldp8008_g_port, ea_ldp8008_g_pin);
		}
			
		clear_pin(ea_ldp8008_s_port, ea_ldp8008_s_pin);
		set_pin  (ea_ldp8008_s_port, ea_ldp8008_s_pin);
		
		set_pins_r_g();
		
		
		if (data.6) { 
			clear_pin(ea_ldp8008_r_port, ea_ldp8008_r_pin);
		}
		if (data.7) {
			clear_pin(ea_ldp8008_g_port, ea_ldp8008_g_pin);
		}
			
		clear_pin(ea_ldp8008_s_port, ea_ldp8008_s_pin);
		set_pin  (ea_ldp8008_s_port, ea_ldp8008_s_pin);
		
		buffer_position0++;
	} 	

#if ea_ldp8008_displays == 2
	#warning "need to work on this"
	for (x = 0; x < 16; x++) {
		data_upper = buffer2[count_byte_disp2];
		data_lower = buffer3[count_byte_disp2];
		
		set_pins_r_g();
		
		if (data_upper.0) { 
			clear_pin(ea_ldp8008_r_port, ea_ldp8008_r_pin);
		}
		if (data_upper.1) {
			clear_pin(ea_ldp8008_g_port, ea_ldp8008_g_pin);
		}
			
		clear_pin(ea_ldp8008_s_port, ea_ldp8008_s_pin);
		set_pin  (ea_ldp8008_s_port, ea_ldp8008_s_pin);
		
		set_pins_r_g();
		
		if (data_upper.2) { 
			clear_pin(ea_ldp8008_r_port, ea_ldp8008_r_pin);
		}
		if (data_upper.3) {
			clear_pin(ea_ldp8008_g_port, ea_ldp8008_g_pin);
		}
		
		clear_pin(ea_ldp8008_s_port, ea_ldp8008_s_pin);
		set_pin  (ea_ldp8008_s_port, ea_ldp8008_s_pin);
		
		set_pins_r_g();
		
		if (data_upper.4) { 
			clear_pin(ea_ldp8008_r_port, ea_ldp8008_r_pin);
		}
		if (data_upper.5) {
			clear_pin(ea_ldp8008_g_port, ea_ldp8008_g_pin);
		}
		
		clear_pin(ea_ldp8008_s_port, ea_ldp8008_s_pin);
		set_pin  (ea_ldp8008_s_port, ea_ldp8008_s_pin);
		
		set_pins_r_g();
		
		
		if (data_upper.6) { 
			clear_pin(ea_ldp8008_r_port, ea_ldp8008_r_pin);
		}
		if (data_upper.7) {
			clear_pin(ea_ldp8008_g_port, ea_ldp8008_g_pin);
		}
			
		clear_pin(ea_ldp8008_s_port, ea_ldp8008_s_pin);
		set_pin  (ea_ldp8008_s_port, ea_ldp8008_s_pin);
		
		count_byte_disp2++;
	} // x loop	
	
#endif
		

	clear_pin(ea_ldp8008_en_port, ea_ldp8008_en_pin);	// turn enable off
	
	clear_pin(ea_ldp8008_a_port, ea_ldp8008_a_pin);
	clear_pin(ea_ldp8008_b_port, ea_ldp8008_b_pin);
	clear_pin(ea_ldp8008_c_port, ea_ldp8008_c_pin);
	if (current_row.0) {
		set_pin(ea_ldp8008_a_port, ea_ldp8008_a_pin);
	}		
	if (current_row.1) {
		set_pin(ea_ldp8008_b_port, ea_ldp8008_b_pin);
	}	
	if (current_row.2) {
		set_pin(ea_ldp8008_c_port, ea_ldp8008_c_pin);
	}		
	// latch data	
	set_pin(ea_ldp8008_l_port, ea_ldp8008_l_pin);
	clear_pin(ea_ldp8008_l_port, ea_ldp8008_l_pin);
	
	// enable display of row
	set_pin(ea_ldp8008_en_port, ea_ldp8008_en_pin);

	current_row++;
	if (current_row == 8) {
		buffer_position0 = 0;
		current_row = 0;
	}	
}

}
uns8 drv_get_pixel(uns8 x, uns8 y) {
	//Should do something to look up the pixel
}

void drv_setup_io() {
	ea_ldp8008_setup_io();
}

void drv_init() {
	ea_ldp8008_init();
}	

#endif
