/*

Copyright (c) 2010, Embedded Adventures, www.embeddedadventures.com
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

- Neither the name of Embedded Adventures nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
THE POSSIBILITY OF SUCH DAMAGE.

Contact us at admin@embeddedadventures.com

*/

/*! \file ds1631.h
    \brief DS1631 temperature sensor routines
	
*/


#include "pic_utils.h"
#include "i2c.h"

#define ds1631_start_convert 0x51
#define ds1631_stop_convert  0x22
#define ds1631_read_temp 0xAA
#define ds1631_access_th 0xA1
#define ds1631_access_tl 0xA2
#define ds1631_access_config 0xAC
#define ds1631_software_por 0x54


/**
	\brief Setup ds1631 ports and pins
	
*/
#define ds1631_setup() ds1631_setup_io()
void ds1631_setup_io(void);

/**
	\brief Set ds1631 config register
	
	Sets the ds1631 config register
	
*/
void ds1631_set_config(uns8 addr, uns8 config);

/**
	\brief Get ds1631 config register
	
	Gets the ds1631 config register (memory location 0x01)


*/
uns8 ds1631_get_config(uns8 addr);


/**
	\brief Read temperature from ds1631

	Returns 16bit raw temperature register from ds1631. Note that if you are
	in one-shot mode (the default) you must have already issued a start_convert
	and waited until it is complete (to check for completion you can either
	wait long enough, or query the config register to check).

*/
uns16 ds1631_get_temp(uns8 addr);

/**
	\brief Start temperature conversion on ds1631
	
	This routine starts the temperature conversion in the ds1631. Issue
	this command before actually reading the temperature.
*/
void ds1631_convert_temp(uns8 addr);
