/*

Copyright (c) 2010, Embedded Adventures, www.embeddedadventures.com
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

- Neither the name of Embedded Adventures nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
THE POSSIBILITY OF SUCH DAMAGE.

Contact us at admin@embeddedadventures.com

*/


#include "its_mode1.h"
#include "its_common.h"
#include "wpan.h"
#include "mrf24j40.h"

#include "config.h"
#include "pic_serial.h"
#include "pic_tick.h"
#include "pic_timer.h"

its1_state state = STATE_STARTUP;

uns8 controller_handle;
uns16 tick_marker;
uns8 channel;

// ---------------------------------------------------------------------------
// Mode 1 common routines 
// ---------------------------------------------------------------------------

void its1_setup_io() {
	wpan_setup_io();
}
	

its1_result its1_controller_handle_association(uns16 pan_id, uns16 its_device_id) {
	// see if this device is in our list, if not, add it
its_device_handle device_handle;

	device_handle = its_get_device(its_device_id);
	
	if (device_handle == ITS_DEVICE_NONE) {
		// in mode 1 we assume that 
		device_handle = its_add_device(/* device */ its_device_id, pan_id, /* short addr */ its_device_id);
		if (device_handle == ITS_DEVICE_NONE) {
			// panic!
			serial_print_str("Too many devices! association failed");
			return RESULT_FAILED;
		} else {
			// send association response
			its_transmit_to_handle(device_handle, ITS_ASSOC_RES, /* nil data */ 0, /* zero length */ 0);
			return RESULT_SUCCESSFUL;
			// although strictly speaking we should wait for the ack...
		}
			
	}
}	



#ifdef ITS_MODE1_CONTROLLER
// controller version
void wpan_data_received_callback(wpan_address *addr, uns8 *data, uns8 data_size) {

serial_print_str("Frame received\n");

uns8 count;
	wpan_print_address(addr);
	//wpan_print_frame_type(frame_type);
	serial_print_str(" Data: ");
	for (count = 0; count < data_size; count++) {
		serial_print_int_hex(data[count]);
		serial_putc(' ');
		if ((data[count] > 31) && (data[count] < 127)) {
			serial_putc(data[count]);
		} else {
			serial_putc('?');
		}	
		serial_putc(' ');
	}
	serial_print_nl();


	
	// see if it is an ITS packet
	if ((data_size > 2) && (data[0] == 'I') && (data[1] == 'T')) {
		serial_print_str("ITS Packet received\n");
		uns8 length_header = data[2];
		uns8 length_data   = data[3+length_header+1];
		uns8 data_start    = 3+length_header+2;
		// okay, but is it an its association request?
		if (data[3] == ITS_ASSOC_REQ) {
			serial_print_str("ITS Association request\n");
			// normally we would check if we are in config mode; 
			// for the moment, let's do this all the time
			uns16 network_id_req = data[6];	//msb of network from header
			network_id_req <<= 8;
			network_id_req += data[5];	//lsb of network from header
			
			if (network_id_req == its_get_network_id()) {	// it's my network
				// it's for me!
				uns16 device_id_req = data[8]; 	// msb source
				device_id_req <<= 8;
				device_id_req += data[7];	// lsb source
				its1_controller_handle_association( /* pan id */network_id_req, device_id_req);
			} else {
				serial_print_str("Not for us, for network: ");
				serial_print_int_hex_16bit(network_id_req);
			}	
		}	// ASSOC REQ
		else if (data[3] == ITS_GENERIC_DATA) {
			// is it for us?
			// who cares right now!	
			// todo...
			serial_print_str("Generic data pkt\n");
			// just issue the callback
			uns16 device_id = data[8];	// msb
			device_id  <<= 8;
			device_id += data[7];	// lsb
			its1_controller_receive_callback(device_id, &data[data_start], data_size - data_start);

		}	// GENERIC DATA		
	} // ITS packet	
} // wpan_frame_received_callback

#else

// device version
void wpan_data_received_callback(wpan_address *addr, uns8 *data, uns8 data_size) {



uns8 count;
	wpan_print_address(addr);
	wpan_print_frame_type(frame_type);
	serial_print_str(" Data: ");
	for (count = 0; count < data_size; count++) {
		serial_print_int_hex(data[count]);
		serial_putc(' ');
		if ((data[count] > 31) && (data[count] < 127)) {
			serial_putc(data[count]);
		} else {
			serial_putc('?');
		}	
		serial_putc(' ');
	}
	serial_print_nl();





	// check frame type
	if ((data_size > 2) && (data[0] == 'I') && (data[1] == 'T')) {
		serial_print_str("ITS Packet received\n");
		uns8 length_header = data[2];
		uns8 length_data   = data[3+length_header+1];
		uns8 data_start    = 3+length_header+2;
		// okay, but is it an its association request?
		if (data[3] == ITS_ASSOC_RES) {
			serial_print_str("Received association response\n");
			if (state == STATE_SEARCHING) {
				serial_print_str("Changing state to associated\n");
				uns16 controller_device_id = data[8]; // device id of controller
				controller_device_id <<= 8;
				controller_device_id += data[7];
				
				uns8 device_handle = its_get_device(controller_device_id);

				if (device_handle == ITS_DEVICE_NONE) {

					device_handle = its_add_device(/* device */ controller_device_id, addr->source_pan_id, controller_device_id);
					if (device_handle == ITS_DEVICE_NONE) {
						// panic!
						serial_print_str("Too many devices! association failed");
						state = STATE_UNASSOCIATED;
					} else {
						controller_handle = device_handle;
						state = STATE_ASSOCIATED;	// stop searching
					}
				} else { // it's already in the table
					state = STATE_ASSOCIATED;
				}	
					
			}	
		}	// association response
		else if (data[3] == ITS_GENERIC_DATA) {
			// is it for us?
			// who cares right now!
			// todo...
			serial_print_str("Generic data pkt\n");
			// just issue the callback
			uns16 device_id = data[8];	// msb
			device_id  <<= 8;
			device_id += data[7];	// lsb
			its1_device_receive_callback(&data[data_start], data_size - data_start);

		}	// GENERIC DATA		
	}
}
#endif

// ---------------------------------------------------------------------------
// Controller specific routines 
// ---------------------------------------------------------------------------

void its1_controller_receive_callback(uns16 device_id, uns8 *data, uns8 data_length);

uns8 its1_controller_init(uns16 my_device_id, uns16 network_id) {
	
uns8 lowest_channel_ed;
uns8 my_ed[8] = ITS_EA;

	its_init();
	wpan_init();
	
	its_set_device_id(my_device_id);
	its_set_network_id(network_id);	// network id = controller id = pan id in mode 1
	
	mrf24j40_set_pan_id(network_id);
	mrf24j40_set_short_address(my_device_id); 	// same as device id
	mrf24j40_set_extended_address(&my_ed);

	serial_print_str("Controller startup\n");
	
	// Find a channel to play on
	serial_print_str("Choosing channel\n");
	
	lowest_channel_ed = mrf24j40_scan_for_lowest_channel_ed();

	serial_print_var("Chosing channel", lowest_channel_ed);
	serial_print_nl();	

	// Now set ourselves up on this channel
	mrf24j40_set_channel(lowest_channel_ed);
	
}	
				
its1_result its1_controller_transmit(uns16 device_id, uns8 *data, uns8 data_length) {

its_device_handle device_handle;

	device_handle = its_get_device(device_id);

	its_transmit_to_handle(device_handle,  ITS_GENERIC_DATA, data, data_length);


}
	



void its1_controller_process() {
}
	

// ---------------------------------------------------------------------------
// Device specific routines 
// ---------------------------------------------------------------------------

// void its1_device_receive_callback(uns8 *data, uns8 data_length);

its1_result its1_device_init(uns16 my_device_id, uns16 network_id) {

// ITS_EA is set in config.h

uns8 my_ea[8] = ITS_EA;

	serial_print_str("Device startup\n");

	its_init();
	wpan_init();
	
	its_set_device_id(my_device_id);
	its_set_network_id(network_id);	// network id = controller id = pan id in mode 1
	
	mrf24j40_set_pan_id(network_id);
	mrf24j40_set_short_address(my_device_id); 	// same as device id
	mrf24j40_set_extended_address(&my_ea);

	state = STATE_UNASSOCIATED;
}

its1_result its1_find_controller() {
	state = STATE_SEARCHING;
	tick_marker = tick_get_count();
	channel = 0;
}

void its1_device_process() {

uns16 test_tick;
uns8 controller_ea[8] = CONTROLLER_EA;

	// are we searching?
	// has time limit expired?
	// go on to next channel
	// send its association request to everyone
	if (state == STATE_SEARCHING) {
		test_tick = tick_get_count();
		if (tick_calc_diff(tick_marker, test_tick) >= 250) {	// 250 = 1/4 second
			tick_marker = test_tick;
			if (channel == 0) { 
				channel = MRF_FIRST_CHANNEL;
			} else {
				if (channel == MRF_LAST_CHANNEL) {
					// Didn't find the controller
					state = STATE_UNASSOCIATED;
					serial_print_str("Didn't find controller\n");
				} else {
					channel++;
				}
			}
			// change channel
			serial_print_var("Trying channel ", channel);
			serial_print_nl();
			mrf24j40_set_channel(channel);
			// send broadcast association request
			its_transmit_to_ea(&controller_ea, 0xffff, ITS_ASSOC_REQ, /* nil data */ 0, /* zero length */ 0);
		} // timer for another channel
	} // we're searching	

}


its1_result its1_device_transmit(uns8 *data, uns8 data_length) {

// We want to transmit to our controller
		its_transmit_to_handle(controller_handle,  ITS_GENERIC_DATA, data, data_length);
}
	
