/*

Copyright (c) 2010, Embedded Adventures, www.embeddedadventures.com
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

- Neither the name of Embedded Adventures nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
THE POSSIBILITY OF SUCH DAMAGE.

Contact us at admin@embeddedadventures.com

*/

/*! \file lm75.h
    \brief LM75 temperature sensor routines

	A library to communicate with the LM75 sensor

*/

#include "pic_utils.h"
#include "i2c.h"

/** Config define for low power mode */
#define LM75_SHUTDOWN	1
/** config define for normal mode */
#define LM75_NORMAL		0



/**
	\brief Setup lm75 ports and pins
	
*/
void lm75_setup(void);

/**
	\brief Set LM75 config register
	
	Sets the LM75 config register (memory location 0x01)
	
*/
void lm75_set_config(uns8 addr, uns8 config);

/**
	\brief Get LM75 config register
	
	Gets the LM75 config register (memory location 0x01)


*/
uns8 lm75_get_config(uns8 addr);


/**
	\brief Request temperature from LM75

	Returns 16bit raw temperature register from LM75

*/

uns16 lm75_get_temp(uns8 addr);
