/*

Copyright (c) 2010, Embedded Adventures, www.embeddedadventures.com
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

- Neither the name of Embedded Adventures nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
THE POSSIBILITY OF SUCH DAMAGE.

Contact us at admin@embeddedadventures.com

*/

/**
	\file pcd8544.h
	\brief PCD8544 Interface routines (used in Nokia 3310 LCD)
	
	Routines to communicate with Nokia 3310 LCD display via the PCD8544 chip
*/

#ifndef __nokia_3310_h
#define __nokia_3310_h

#include "config.h"



#include "pic_utils.h"


/* Include this in your config.h

// - - - - - - - - - - - - - - - - - - - - 
// PCD8544 (Nokia 3310) interface defines
// - - - - - - - - - - - - - - - - - - - - 

#define pcd8544_res_port  PORTD
#define pcd8544_res_pin   3

#define pcd8544_sclk_port  PORTC
#define pcd8544_sclk_pin   3

#define pcd8544_sdin_port PORTC
#define pcd8544_sdin_pin  5

#define pcd8544_dc_port   PORTF
#define pcd8544_dc_pin    6

#define pcd8544_sce_port  PORTF
#define pcd8544_sce_pin   7
*/


#define pcd8544_setup() pcd8544_setup_io()
void pcd8544_setup_io();
void pcd8544_init();
void pcd8544_send_byte(uns8 b);
void pcd8544_write(uns8 mem_addr, uns8 data);
void pcd8544_send_command(uns8 command);
void pcd8544_send_data(uns8 data);	
void pcd8544_set_pixel(uns8 x, uns8 y, uns8 colour);
void pcd8544_clear();

// - - - - - - - - - - - - - - - - - - - - 
#ifdef fix_this_later

#ifndef i2c_scl_port
	#error "You haven't defined i2c_scl_port in your config.h!"
#endif

#ifndef i2c_sda_port
	#error "You haven't defined i2c_sda_port in your config.h!"
#endif

#ifndef i2c_scl_pin
	#error "You haven't defined i2c_scl_pin in your config.h!"
#endif

#ifndef i2c_sda_pin
	#error "You haven't defined i2c_sda_pin in your config.h!"
#endif
#endif


#endif