/*

Copyright (c) 2010, Embedded Adventures, www.embeddedadventures.com
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

- Neither the name of Embedded Adventures nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
THE POSSIBILITY OF SUCH DAMAGE.

Contact us at admin@embeddedadventures.com

*/

/*! \file platform_leds.h
    \brief Easy access to the LEDs on your platform
	
*/
#include "platform.h"

#ifndef __platform_leds_h
#define __platform_heds_h

#include "pic_utils.h"

#if PLATFORM_TYPE == EA_WEATHER_STATION
	#define LEDS_DEFINED
	#define PLATFORM_LEDS_COUNT	3
	#define led1_port	PORTA
	#define led1_pin	0
	#define led2_port	PORTA
	#define led2_pin	1
	#define led3_port	PORTA
	#define led3_pin	2
#endif

#if PLATFORM_TYPE == TIRTLE_BOARD
	#define LEDS_DEFINED
	#define led1_port	PORTA
	#define led1_pin	0
	#define led2_port	PORTA
	#define led2_pin	1
	#define led3_port	PORTA
	#define led3_pin	2
	
#endif

#if PLATFORM_TYPE == EA_WIRELESS_TEMP_SENSOR
	#define LEDS_DEFINED
	#define led1_port	PORTA
	#define led1_pin	0
	#define led2_port	PORTA
	#define led2_pin	1
	#define led3_port	PORTA
	#define led3_pin	2
#endif

#if PLATFORM_TYPE == EA_LED_PANEL_DRIVER
	#define LEDS_DEFINED
	#define led1_port	PORTC
	#define led1_pin	1
#endif

#if PLATFORM_TYPE == EA_USB2SERIAL
	#define LEDS_DEFINED
	#define led1_port	PORTC
	#define led1_pin	5
	#define led2_port	PORTC
	#define led2_pin	7
	#define led3_port	PORTC
	#define led3_pin	6
#endif

#if PLATFORM_TYPE == OLIMEX_BOARD
	#define LEDS_DEFINED
	#define led1_port	PORTA
	#define led1_pin	0
#endif


#ifdef led1_port
		#define platform_led1_on() \
			set_pin(led1_port, led1_pin)
		
		#define platform_led1_off() \
			clear_pin(led1_port, led1_pin)
		
#endif
#ifdef led2_port
		#define platform_led2_on() \
			set_pin(led2_port, led2_pin)
		
		#define platform_led2_off() \
			clear_pin(led2_port, led2_pin)
		
#endif
#ifdef led3_port
		#define platform_led3_on() \
			set_pin(led3_port, led3_pin)
		
		#define platform_led3_off() \
			clear_pin(led3_port, led3_pin)
		
#endif

void platform_leds_setup_io();
void platform_leds_flash(uns8 led);
void platform_leds_flashing(uns8 led, uns8 enable);
void platform_leds_on(uns8 led);
void platform_leds_off(uns8 led);
void platform_leds_process();

#ifndef LEDS_DEFINED
	#warning "Don't know about the leds on your platform! See platform_leds.h"
#endif


#endif
