/*

Copyright (c) 2010, Embedded Adventures, www.embeddedadventures.com
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

- Neither the name of Embedded Adventures nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
THE POSSIBILITY OF SUCH DAMAGE.

Contact us at admin@embeddedadventures.com

*/

/*! \file sht15.h
    \brief Support for SHT15 and SHT11 digital humidity sensors
	
// Include the following in your config.h:

// - - - - - - - - - - - 
// sht15 defines
// - - - - - - - - - - -

#define sht15_sck_port PORTA
#define sht15_sck_pin 1
#define sht15_sda_port PORTA
#define sht15_sda_pin 0

// - - - - - - - - - - -	
	
*/


#ifndef __sht15_h
#define __sht15_h
#include "pic_utils.h"
#include "config.h"




void sht15_setup_io(void);

void sht15_start(void);
void sht15_read(void);
uns16 sht15_read_humidity(void);
uns16 sht15_read_temperature(void);
void sht15_send_byte(uns8 sht15_command);
uns16 sht15_read_byte16(void);
uns16 sht15_fix_humidity(uns16 sensor_out);
uns16 sht15_fix_humidity_r(uns16 sensor_out);
int16 sht15_fix_temperature_h(uns16 sensor_out);
uns16 sht15_fix_humidity_l(uns8 sensor_out);

#ifndef sht15_sck_port
	#error "sht15_sck_port not defined in config.h"
#endif	
#ifndef sht15_sda_port
	#error "sht15_sda_port not defined in config.h"
#endif	
#ifndef sht15_sck_pin
	#error "sht15_sck_pin not defined in config.h"
#endif	
#ifndef sht15_sda_pin
	#error "sht15_sda_pin not defined in config.h"
#endif	



#endif