//-----------------------------------------------------
// Pic Pack library
// 
// clock.c
//
// It's a clock! Using PLT-1003 USB2Serial platform,
// DSP-7S04 7 segment display and MOD-1005 real time clock
//
// www.embeddedadventures.com
//
//-----------------------------------------------------


// Pic Pack includes
#include "config.h"
#include "pic_utils.h"
#include "pic_serial.h"
#include "cat4016.h"
#include "ea_dsp7s04.h"
#include "m41t81s.h"

// System includes
#include "string.h"
#include "stdlib.h"
#include "config.h"
#include "pic_term.h"


uns8 rtc_int = 0;
uns8 button0 = 0;
uns8 button1 = 0;

// Interrupt routine - - - - - - - - - -

void interrupt() {
	
	serial_handle_tx_isr();
	serial_handle_rx_isr();

	if (test_bit(intcon, INT0IF)) {
		clear_bit(intcon, INT0IF);
		button0 = 1;
	};

	if (test_bit(intcon3, INT1IF)) {
		clear_bit(intcon3, INT1IF);
		button1 = 1;
	};
		

}



void update_display(void) {
	
uns8 minutes, hours, seconds, day, date, month, year;
char buffer[10];

	minutes = rtc_get_minutes();
	hours   = rtc_get_hours();
	seconds = rtc_get_seconds();
	day 	= rtc_get_dow();
	date 	= rtc_get_date();
	month 	= rtc_get_month();
	year 	= rtc_get_year();
	
	if (hours > 9) {
		buffer[0] = '0' + hours / 10;
		buffer[1] = '0' + hours % 10;
	} else {
		buffer[0] = ' ';
		buffer[1] = '0' + hours;
	}
	
	buffer[2] = '0' + minutes / 10;
	buffer[3] = '0' + minutes % 10;
	buffer[4] = 0;
	
	ea_dsp7s04_print_str(buffer);
	ea_dsp7s04_update_display();
}


//
// term_entry_callback will be called when 
// user pressed <enter> on serial "terminal".
// term_buffer contains the characters (if any)
// that the user has entered.

void term_entry_callback(uns8 *term_buffer) {

uns8 count;
uns8 display[5];


	if (term_buffer[0] == '$') {
		rtc_start_clock();
	}	
	if (term_buffer[0] == '!') {
		rtc_int = 1;
	} else {
		if (strlen(term_buffer) < 4) {
			serial_print_str("Not enough characters! Type 4 characters and press enter\n");
		} else {	
			strncpy(display, term_buffer, 4);
			display[4] = '\0';
			ea_dsp7s04_print_str(display);
		}	
	}

}	// term_entry_callback


// configure_system
//
// Get the pic ready for everything we want to do

void configure_system() {
	
	
	// turn off interrupts just in case
	kill_interrupts();
	
	// kill those pesky analogue inputs
	turn_analog_inputs_off();	
	
	// set serial to run at 115200bps 
	serial_setup(SPBRG_115200);
	
	// initialise termainl
	term_init();
	
	ea_dsp7s04_setup_io();
	ea_dsp7s04_init();
	make_input(rtc_int_port, rtc_int_pin);	// sqw wired to this pin
	
	rtc_setup_io();
	
	
	// the following assumes it's RA0
	set_bit(ioca, IOCA0);	// ARRRRRG magic required here

	clear_bit(intcon2, INTEDG0);
	clear_bit(intcon2, INTEDG1);
	clear_bit(intcon3, INT1IF);
	clear_bit(intcon, INT0IF);
	set_bit(intcon, INT0IE);
	set_bit(intcon3, INT1IE);
	
	
	// turn peripheral interrupts on (ie, serial port)
	turn_peripheral_ints_on();
	
	// turn all interrupts on
	turn_global_ints_on();

}

uns8 dot_display = 0;

void main() {
	
	configure_system();
	
	delay_ms(100);	// mostly so screamer bootloader has time to switch windows


	uns8 reg1 = i2c_read_eeprom(m41t81s_device_addr, m41t81s_seconds_reg);
	if (test_bit(reg1, 7)) {
		serial_print_str(" STOPPED ");
	} else {
			serial_print_str(" STARTED ");
	}
	uns8 reg0c = i2c_read_eeprom(m41t81s_device_addr, m41t81s_alarm_hour_reg);
	if (test_bit(reg0c, 6)) {
		serial_print_str(" HALTED ");
	} else {
			serial_print_str(" NOT HALTED	 ");
	}
	uns8 h = rtc_get_hours();
	uns8 m = rtc_get_minutes();
	serial_print_str("Time = ");
	serial_print_int(h);
	serial_putc(':');
	serial_print_int(m);
	serial_print_nl();
			
	for (uns8 count = 0; count < 0x14; count++) {
		
		serial_print_int_hex(count);
		serial_print_str("=");
		serial_print_int_hex(i2c_read_eeprom(m41t81s_device_addr, count));
		serial_print_nl();
	}	
	
	
//	rtc_stop_clock();
	rtc_start_clock();	// unpause the m41t81s
	rtc_set_sqw_freq(rtc_sqw_freq_2Hz);	// give us a shove every half second
	rtc_start_sqw_output();	// turn on the int output
	
	reg1 = i2c_read_eeprom(m41t81s_device_addr, m41t81s_seconds_reg);
	if (test_bit(reg1, 7)) {
		serial_print_str(" STOPPED ");
	} else {
			serial_print_str(" STARTED ");
	}
	reg0c = i2c_read_eeprom(m41t81s_device_addr, m41t81s_alarm_hour_reg);
	if (test_bit(reg0c, 6)) {
		serial_print_str(" HALTED ");
	} else {
			serial_print_str(" NOT HALTED	 ");
	}
	h = rtc_get_hours();
	m = rtc_get_minutes();
	serial_print_str("Time = ");
	serial_print_int(h);
	serial_putc(':');
	serial_print_int(m);
	serial_print_nl();
	
	serial_print_str("\n\nCLOCK\nn");
	serial_print_str( "\n<");
	
	// Print the time and date this file was compiled. Handy to know which
	// version you are running.
	serial_print_str(__TIME__);
	serial_putc(' ');
	serial_print_str(__DATE__);
	serial_print_str(">\n");

	ea_dsp7s04_enable_display(1);
	
	dot_display = 0;
	uns8 last_check = 0;
	
	for(;;) {

		term_process();
		
		if (button0) {
			serial_print_str("H");
			uns8 hours = rtc_get_hours();
			hours++;
			if (hours > 23) {
				hours = 0;
			}
			rtc_set_hours(hours);
			button0 = 0;
			update_display();	
		}
		if (button1) {
			serial_print_str("M");
			uns8 mins = rtc_get_minutes();
			mins++;
			if (mins > 59) {
				mins = 0;
			}
			rtc_set_minutes(mins);
			rtc_set_seconds(0);
			button1 = 0;
			update_display();	
		}
		
		if (test_pin(rtc_int_port, rtc_int_pin)) {
			
			if (last_check == 0) {
				rtc_int = 1;
				last_check = 1;
			}
		} else {
			last_check = 0;
		}	
			
		if (rtc_int) {
			rtc_int = 0;
			if (dot_display) {
				ea_dsp7s04_clear_dot(1);
			} else {
				ea_dsp7s04_set_dot(1);
			}
			dot_display	= ~dot_display;
			update_display();
		}	
	}

	
}	// main

